@@ -1,18 +1,18 @@
-// 
-// Copyright (c) 2010-2012, Benjamin Kaufmann
-// 
-// This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
-// 
+//
+// Copyright (c) 2010-2015, Benjamin Kaufmann
+//
+// This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/
+//
 // Clasp is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 2 of the License, or
 // (at your option) any later version.
-// 
+//
 // Clasp is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU General Public License
 // along with Clasp; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
@@ -34,49 +34,71 @@
 #include <clasp/solver_types.h>
 
 /*!
- * \file 
- * Defines classes controlling multi-threaded parallel solving.
- * 
+ * \file
+ * \brief Defines classes controlling multi-threaded parallel solving.
  */
-namespace Clasp { namespace mt {
+namespace Clasp {
+//! Namespace for types and functions needed for implementing multi-threaded parallel solving.
+namespace mt {
+
+/**
+ * \defgroup mt Multi-threading
+ * \brief Parallel solving and related classes.
+ * \ingroup enumerator
+ */
+//@{
 
 class ParallelHandler;
 class ParallelSolve;
 
+//! Options for controlling parallel solving.
 struct ParallelSolveOptions : BasicSolveOptions {
-	typedef Distributor::Policy Distribution;
+	//! Nogood distribution options.
+	struct Distribution : Distributor::Policy {
+		enum Mode { mode_global = 0, mode_local = 1 };
+		Distribution(Mode m = mode_global) : Distributor::Policy(), mode(m) {}
+		Distributor::Policy& policy() { return *this; }
+		uint32 mode;
+	};
 	ParallelSolveOptions() {}
+	//! Algorithm options.
 	struct Algorithm {
+		//! Possible search strategies.
 		enum SearchMode { mode_split = 0, mode_compete  = 1 };
 		Algorithm() : threads(1), mode(mode_compete) {}
 		uint32     threads;
 		SearchMode mode;
 	};
-	struct Integration { /**< Nogood integration options. */
+	//! Nogood integration options.
+	struct Integration {
 		static const uint32 GRACE_MAX = (1u<<28)-1;
 		Integration() :  grace(1024), filter(filter_gp), topo(topo_all) {}
 		enum Filter   { filter_no = 0, filter_gp = 1, filter_sat = 2, filter_heuristic = 3 };
 		enum Topology { topo_all  = 0, topo_ring = 1, topo_cube  = 2, topo_cubex = 3       };
 		uint32 grace : 28; /**< Lower bound on number of shared nogoods to keep. */
-		uint32 filter: 2;  /**< Filter for integrating shared nogoods (one of Filter). */   
+		uint32 filter: 2;  /**< Filter for integrating shared nogoods (one of Filter). */
 		uint32 topo  : 2;  /**< Integration topology */
 	};
-	struct GRestarts {   /**< Options for configuring global restarts. */
+	//! Global restart options.
+	struct GRestarts {
 		GRestarts():maxR(0) {}
 		uint32           maxR;
 		ScheduleStrategy sched;
 	};
-	Integration  integrate; /**< Nogood integration parameters.     */
-	Distribution distribute;/**< Nogood distribution parameters.    */
-	GRestarts    restarts;  /**< Global restart strategy.           */
-	Algorithm    algorithm; /**< Parallel algorithm to use.         */
+	Integration  integrate; //!< Nogood integration options to apply during search.
+	Distribution distribute;//!< Nogood distribution options to apply during search.
+	GRestarts    restarts;  //!< Global restart strategy to apply during search.
+	Algorithm    algorithm; //!< Parallel algorithm to use.
 	//! Allocates a new solve object.
 	SolveAlgorithm* createSolveObject() const;
 	//! Returns the number of threads that can run concurrently on the current hardware.
-	static uint32   recommendedSolvers()     { return Clasp::thread::hardware_concurrency(); }
+	static uint32   recommendedSolvers()     { return Clasp::mt::thread::hardware_concurrency(); }
 	//! Returns number of maximal number of supported threads.
 	static uint32   supportedSolvers()       { return 64; }
+	//! Returns the peers of the solver with the given id assuming the given topology.
+	static uint64   initPeerMask(uint32 sId, Integration::Topology topo, uint32 numThreads);
 	uint32          numSolver()        const { return algorithm.threads; }
+	void            setSolvers(uint32 i)     { algorithm.threads = std::max(uint32(1), i); }
 	bool            defaultPortfolio() const { return algorithm.mode == Algorithm::mode_compete; }
 };
 
@@ -90,7 +112,7 @@ struct ParallelSolveOptions : BasicSolveOptions {
  */
 class ParallelSolve : public SolveAlgorithm {
 public:
-	explicit ParallelSolve(Enumerator* e, const ParallelSolveOptions& opts);
+	explicit ParallelSolve(const ParallelSolveOptions& opts);
 	~ParallelSolve();
 	// base interface
 	virtual bool interrupted() const;
@@ -118,17 +140,20 @@ private:
 	enum ErrorCode { error_none = 0, error_oom = 1, error_runtime = 2, error_other = 4 };
 	enum           { masterId = 0 };
 	// -------------------------------------------------------------------------------------------
-	// Thread setup 
+	// Thread setup
 	struct EntryPoint;
 	void   destroyThread(uint32 id);
-	void   allocThread(uint32 id, Solver& s, const SolveParams& p); 
+	void   allocThread(uint32 id, Solver& s, const SolveParams& p);
 	void   joinThreads();
 	// -------------------------------------------------------------------------------------------
 	// Algorithm steps
 	void   setIntegrate(uint32 grace, uint8 filter);
 	void   setRestarts(uint32 maxR, const ScheduleStrategy& rs);
-	bool   beginSolve(SharedContext& ctx);
+	bool   beginSolve(SharedContext& ctx, const LitVec& assume);
 	bool   doSolve(SharedContext& ctx, const LitVec& assume);
+	void   doStart(SharedContext& ctx, const LitVec& assume);
+	int    doNext(int last);
+	void   doStop();
 	bool   doInterrupt();
 	void   solveParallel(uint32 id);
 	void   initQueue();
@@ -137,6 +162,7 @@ private:
 	bool   waitOnSync(Solver& s);
 	void   exception(uint32 id, PathPtr& path, ErrorCode e, const char* what);
 	void   reportProgress(const Event& ev) const;
+	void   reportProgress(const Solver& s, const char* msg) const;
 	// -------------------------------------------------------------------------------------------
 	typedef ParallelSolveOptions::Distribution Distribution;
 	struct SharedData;
@@ -145,16 +171,16 @@ private:
 	ParallelHandler** thread_;       // Thread-locl control data
 	// READ ONLY
 	Distribution      distribution_; // distribution options
-	uint32            maxRestarts_;  // disable global restarts once reached 
+	uint32            maxRestarts_;  // disable global restarts once reached
 	uint32            intGrace_ : 30;// grace period for clauses to integrate
 	uint32            intTopo_  :  2;// integration topology
 	uint32            intFlags_;     // bitset controlling clause integration
-	GpType            initialGp_;
+	bool              modeSplit_;
 };
-
+//! An event type for debugging messages sent between threads.
 struct MessageEvent : SolveEvent<MessageEvent> {
 	enum Action { sent, received, completed };
-	MessageEvent(const Solver& s, const char* message, Action a, double t = 0.0) 
+	MessageEvent(const Solver& s, const char* message, Action a, double t = 0.0)
 		: SolveEvent<MessageEvent>(s, verbosity_high), msg(message), time(t) { op = (uint32)a; }
 	const char* msg;    // name of message
 	double      time;   // only for action completed
@@ -185,13 +211,13 @@ public:
 	int  error() const   { return (int)error_; }
 	void setWinner()     { win_ = 1; }
 	bool winner() const  { return win_ != 0; }
-	void setThread(Clasp::thread& x) { assert(!joinable()); x.swap(thread_); assert(joinable()); }
-	
+	void setThread(Clasp::mt::thread& x) { assert(!joinable()); x.swap(thread_); assert(joinable()); }
+
 	//! True if *this has an associated thread of execution, false otherwise.
 	bool joinable() const { return thread_.joinable(); }
 	//! Waits for the thread of execution associated with *this to finish.
 	int join() { if (joinable()) { thread_.join(); } return error(); }
-	
+
 	// overridden methods
 
 	//! Integrates new information.
@@ -203,15 +229,15 @@ public:
 	bool isModel(Solver& s);
 
 	// own interface
-	
-	// TODO: make functions virtual once necessary 
-	
+
+	// TODO: make functions virtual once necessary
+
 	//! Returns true if handler's guiding path is disjoint from all others.
 	bool disjointPath() const { return gp_.type == ParallelSolve::gp_split; }
 	//! Returns true if handler has a guiding path.
 	bool hasPath()      const { return gp_.type != ParallelSolve::gp_none; }
 	void setGpType(GpType t)  { gp_.type = t; }
-	
+
 	//! Entry point for solving the given guiding path.
 	/*!
 	 * \param solve   The object used for solving.
@@ -222,12 +248,12 @@ public:
 
 	/*!
 	 * \name Message handlers
-	 * \note 
+	 * \note
 	 *   Message handlers are intended as callbacks for ParallelSolve::handleMessages().
 	 *   They shall not change the assignment of the solver object.
 	 */
 	//@{
-	
+
 	//! Algorithm is about to terminate.
 	/*!
 	 * Removes this object from the solver's list of post propagators.
@@ -249,23 +275,22 @@ public:
 
 	Solver&            solver()      { return *solver_; }
 	const SolveParams& params() const{ return *params_; }
-	//@}  
+	//@}
 private:
 	void add(ClauseHead* h);
 	void clearDB(Solver* s);
 	bool integrate(Solver& s);
-	typedef LitVec::size_type size_type;
 	typedef PodVector<Constraint*>::type ClauseDB;
 	typedef SharedLiterals**             RecBuffer;
 	enum { RECEIVE_BUFFER_SIZE = 32 };
 	ParallelSolve*     ctrl_;       // my message source
 	Solver*            solver_;     // my solver
 	const SolveParams* params_;     // my solving params
-	Clasp::thread      thread_;     // active thread or empty for master
+	Clasp::mt::thread  thread_;     // active thread or empty for master
 	RecBuffer          received_;   // received clauses not yet integrated
 	ClauseDB           integrated_; // my integrated clauses
-	size_type          recEnd_;     // where to put next received clause
-	size_type          intEnd_;     // where to put next clause
+	uint32             recEnd_;     // where to put next received clause
+	uint32             intEnd_;     // where to put next clause
 	uint32             error_:30;   // error code or 0 if ok
 	uint32             win_  : 1;   // 1 if thread was the first to terminate the search
 	uint32             up_   : 1;   // 1 if next propagate should check for new lemmas/models
@@ -281,39 +306,62 @@ private:
 		}
 	} gp_;
 };
-
-class GlobalQueue : public Distributor {
+//! A class that uses a global list to exchange nogoods between threads.
+class GlobalDistribution : public Distributor {
 public:
-	explicit GlobalQueue(const Policy& p, uint32 maxShare, uint32 topo);
-	~GlobalQueue();
+	explicit GlobalDistribution(const Policy& p, uint32 maxShare, uint32 topo);
+	~GlobalDistribution();
 	uint32  receive(const Solver& in, SharedLiterals** out, uint32 maxOut);
 	void    publish(const Solver& source, SharedLiterals* n);
 private:
 	void release();
-	struct DistPair {
-		DistPair(uint32 sId = UINT32_MAX, SharedLiterals* x = 0) : sender(sId), lits(x) {}
+	struct ClausePair {
+		ClausePair(uint32 sId = UINT32_MAX, SharedLiterals* x = 0) : sender(sId), lits(x) {}
 		uint32          sender;
 		SharedLiterals* lits;
 	};
-	class Queue : public MultiQueue<DistPair> {
+	class Queue : public MultiQueue<ClausePair> {
 	public:
-		typedef MultiQueue<DistPair> base_type;
+		typedef MultiQueue<ClausePair> base_type;
 		using base_type::publish;
 		Queue(uint32 m) : base_type(m) {}
 	};
 	struct ThreadInfo {
-		Queue::ThreadId id;
 		uint64          peerMask;
-		char pad[64 - sizeof(Queue::ThreadId)];
+		union {
+			Queue::ThreadId id;
+			uint64          rep;
+		};
+		char            pad[64 - (sizeof(uint64)*2)];
 	};
 	Queue::ThreadId& getThreadId(uint32 sId) const { return threadId_[sId].id; }
 	uint64           getPeerMask(uint32 sId) const { return threadId_[sId].peerMask; }
-	uint64           populatePeerMask(uint32 topo, uint32 maxT, uint32 id) const;
-	uint64           populateFromCube(uint32 maxT, uint32 id, bool ext) const;
 	Queue*           queue_;
 	ThreadInfo*      threadId_;
 };
-
+//! A class that uses thread-local lists to exchange nogoods between threads.
+class LocalDistribution : public Distributor {
+public:
+	explicit LocalDistribution(const Policy& p, uint32 maxShare, uint32 topo);
+	~LocalDistribution();
+	uint32  receive(const Solver& in, SharedLiterals** out, uint32 maxOut);
+	void    publish(const Solver& source, SharedLiterals* n);
+private:
+	typedef Detail::RawStack   RawStack;
+	typedef MPSCPtrQueue::Node QNode;
+	enum { BLOCK_CAP = 128 };
+	QNode* allocNode(uint32 tId, SharedLiterals* clause);
+	void   freeNode(uint32  tId, QNode* n) const;
+	struct ThreadData {
+		MPSCPtrQueue received; // queue holding received clauses
+		uint64       peers;    // set of peers from which this thread receives clauses
+		QNode        sentinal; // sentinal node for simplifying queue impl
+		QNode*       free;     // local free list - only accessed by this thread
+	}**            thread_;    // one entry for each thread
+	RawStack       blocks_;    // allocated node blocks
+	uint32         numThread_; // number of threads, i.e. size of array thread_
+};
+//@}
 } }
 #endif
 
