@@ -1,18 +1,18 @@
-// 
+//
 // Copyright (c) 2010, Benjamin Kaufmann
-// 
-// This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
-// 
+//
+// This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/
+//
 // Clasp is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 2 of the License, or
 // (at your option) any later version.
-// 
+//
 // Clasp is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU General Public License
 // along with Clasp; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
@@ -25,7 +25,7 @@
 #pragma once
 #endif
 #include <clasp/solver.h>
-#include <clasp/literal.h> 
+#include <clasp/literal.h>
 #include <clasp/dependency_graph.h>
 #include <clasp/constraint.h>
 namespace Clasp {
@@ -33,8 +33,9 @@ class LoopFormula;
 
 //! Clasp's default unfounded set checker.
 /*!
- * \ingroup constraint
+ * \ingroup propagator
  * Searches for unfounded atoms by checking the positive dependency graph (PDG)
+ *
  * Basic Idea:
  *  - For each (non-false) atom a, let source(a) be a body B in body(a) that provides an external support for a
  *    - If no such B exists, a must be false
@@ -47,26 +48,29 @@ class LoopFormula;
  */
 class DefaultUnfoundedCheck : public PostPropagator {
 public:
-	typedef SharedDependencyGraph DependencyGraph;
+	typedef Asp::PrgDepGraph           DependencyGraph;
 	typedef DependencyGraph::NodeId    NodeId;
 	typedef DependencyGraph::BodyNode  BodyNode;
 	typedef DependencyGraph::AtomNode  AtomNode;
+	typedef const DependencyGraph*     ConstGraphPtr;
+	typedef DependencyGraph*           GraphPtr;
 	//! Defines the supported reasons for explaining assignments.
 	enum ReasonStrategy {
-		common_reason,    /*!< one reason for each unfounded set but one clause for each atom */
+		common_reason = LoopReason_t::Explicit, /*!< one reason for each unfounded set but one clause for each atom */
+		only_reason   = LoopReason_t::Implicit, /*!< store only the reason but don't learn a nogood */
 		distinct_reason,  /*!< distinct reason and clause for each unfounded atom */
 		shared_reason,    /*!< one shared loop formula for each unfounded set */
-		only_reason,      /*!< store only the reason but don't learn a nogood */
 		no_reason,        /*!< do no compute reasons for unfounded sets (only valid if learning is disabled!) */
 	};
-	
-	DefaultUnfoundedCheck();
+
+	explicit DefaultUnfoundedCheck(DependencyGraph& graph, ReasonStrategy st = common_reason);
 	~DefaultUnfoundedCheck();
 
 	ReasonStrategy reasonStrategy() const { return strategy_; }
-	
-	DependencyGraph* graph() const { return graph_; }
-	uint32           nodes() const { return static_cast<uint32>(atoms_.size() + bodies_.size()); }
+	void           setReasonStrategy(ReasonStrategy rs);
+
+	ConstGraphPtr graph() const { return graph_; }
+	uint32        nodes() const { return static_cast<uint32>(atoms_.size() + bodies_.size()); }
 
 	// base interface
 	uint32 priority() const { return uint32(priority_reserved_ufs); }
@@ -75,6 +79,8 @@ public:
 	bool   propagateFixpoint(Solver& s, PostPropagator* ctx);
 	bool   isModel(Solver& s);
 	bool   valid(Solver& s);
+	bool   simplify(Solver& s, bool);
+	void   destroy(Solver* s, bool detach);
 private:
 	DefaultUnfoundedCheck(const DefaultUnfoundedCheck&);
 	DefaultUnfoundedCheck& operator=(const DefaultUnfoundedCheck&);
@@ -103,7 +109,7 @@ private:
 	};
 	// data for extended bodies
 	struct ExtData {
-		ExtData(weight_t bound, uint32 preds) : lower(bound) {
+		ExtData(weight_t bound, uint32 preds) : lower(bound), slack(-bound) {
 			for (uint32 i = 0; i != flagSize(preds); ++i) { flags[i] = 0; }
 		}
 		bool addToWs(uint32 idx, weight_t w) {
@@ -126,7 +132,10 @@ private:
 		}
 		static   uint32 flagSize(uint32 preds) { return (preds+31)/32; }
 		weight_t lower;
+		weight_t slack;
+CLASP_WARNING_BEGIN_RELAXED
 		uint32   flags[0];
+CLASP_WARNING_END_RELAXED
 	};
 	// data for each atom
 	struct AtomData {
@@ -165,8 +174,9 @@ private:
 		uint32   high;
 		uint32   low;
 		uint32   next;
+		uint32   scc;
 	};
-	// -------------------------------------------------------------------------------------------  
+	// -------------------------------------------------------------------------------------------
 	// constraint interface
 	PropResult propagate(Solver&, Literal, uint32& data) {
 		uint32 index = data >> 2;
@@ -186,8 +196,9 @@ private:
 	void    addWatch(Literal, uint32 data, WatchType type);
 	void    addExtWatch(Literal p, const BodyPtr& n, uint32 data);
 	struct  InitExtWatches {
-		void operator()(Literal p, uint32 idx, bool ext) const { 
-			self->addExtWatch(~p, *B, (idx<<1)+uint32(ext)); 
+		void operator()(Literal p, uint32 idx, bool ext) const {
+			extra->slack += B->node->pred_weight(idx, ext);
+			self->addExtWatch(~p, *B, (idx<<1)+uint32(ext));
 			if (ext && !self->solver_->isFalse(p)) {
 				extra->addToWs(idx, B->node->pred_weight(idx, true));
 			}
@@ -196,7 +207,12 @@ private:
 		const BodyPtr*         B;
 		ExtData*               extra;
 	};
-	// -------------------------------------------------------------------------------------------  
+	struct RemExtWatches {
+		void operator()(Literal p, uint32, bool) const { s->removeWatch(~p, self); }
+		Constraint* self;
+		Solver*     s;
+	};
+	// -------------------------------------------------------------------------------------------
 	// propagating source pointers
 	void propagateSource();
 	struct AddSource { // an atom in a body has a new source, check if body is now a valid source
@@ -210,12 +226,12 @@ private:
 		void operator()(NodeId bId, uint32 idx) const;
 		DefaultUnfoundedCheck* self;
 	};
-	struct RemoveSource {// an atom in a body has lost its source, check if body is no longer a valid source 
+	struct RemoveSource {// an atom in a body has lost its source, check if body is no longer a valid source
 		explicit RemoveSource(DefaultUnfoundedCheck* u, bool add = false) : self(u), addTodo(add) {}
 		// normal body
-		void operator()(NodeId bId) const { 
-			if (++self->bodies_[bId].lower_or_ext == 1 && self->bodies_[bId].watches != 0) { 
-				self->forwardUnsource(self->getBody(bId), addTodo); 
+		void operator()(NodeId bId) const {
+			if (++self->bodies_[bId].lower_or_ext == 1 && self->bodies_[bId].watches != 0) {
+				self->forwardUnsource(self->getBody(bId), addTodo);
 			}
 		}
 		// extended body
@@ -228,7 +244,7 @@ private:
 	void forwardSource(const BodyPtr& n);
 	void forwardUnsource(const BodyPtr& n, bool add);
 	void updateSource(AtomData& atom, const BodyPtr& n);
-	// -------------------------------------------------------------------------------------------  
+	// -------------------------------------------------------------------------------------------
 	// finding & propagating unfounded sets
 	void updateAssignment(Solver& s);
 	bool findSource(NodeId atom);
@@ -238,32 +254,40 @@ private:
 	bool assertAtom(Literal a, UfsType t);
 	void computeReason(UfsType t);
 	void addIfReason(const BodyPtr&, uint32 uScc);
+	bool isExternal(const BodyPtr&, weight_t& slack) const;
 	void addDeltaReason(const BodyPtr& body, uint32 uScc);
 	void addReasonLit(Literal);
 	void createLoopFormula();
 	struct  AddReasonLit {
-		void operator()(Literal p, NodeId, bool) const { if (self->solver_->isFalse(p)) self->addReasonLit(p); }
+		void operator()(Literal p, NodeId id, bool ext) const {
+			if (self->solver_->isFalse(p) && slack >= 0) {
+				slack -= node->pred_weight(id, ext);
+				self->addReasonLit(p);
+			}
+		}
 		DefaultUnfoundedCheck* self;
+		const BodyNode*        node;
+		mutable weight_t       slack;
 	};
 	UfsType findUfs(Solver& s, bool checkNonHcf);
 	UfsType findNonHcfUfs(Solver& s);
-	// -------------------------------------------------------------------------------------------  
+	// -------------------------------------------------------------------------------------------
 	bool pushTodo(NodeId at) { return (atoms_[at].todo == 0 && (todo_.push(at), atoms_[at].todo = 1) != 0); }
 	bool pushUfs(NodeId at)  { return (atoms_[at].ufs  == 0 && (ufs_.push(at),  atoms_[at].ufs  = 1) != 0); }
 	void resetTodo()         { while (!todo_.empty()){ atoms_[todo_.pop_ret()].todo = 0; } todo_.clear();   }
-	void resetUfs()          { while (!ufs_.empty()) { atoms_[ufs_.pop_ret()].ufs   = 0; } ufs_.clear();    } 
-	// -------------------------------------------------------------------------------------------  
+	void resetUfs()          { while (!ufs_.empty()) { atoms_[ufs_.pop_ret()].ufs   = 0; } ufs_.clear();    }
+	// -------------------------------------------------------------------------------------------
 	typedef PodVector<AtomData>::type       AtomVec;
 	typedef PodVector<BodyData>::type       BodyVec;
 	typedef PodVector<ExtData*>::type       ExtVec;
 	typedef PodVector<ExtWatch>::type       WatchVec;
 	typedef PodQueue<NodeId>                IdQueue;
 	typedef SingleOwnerPtr<MinimalityCheck> MiniPtr;
-	// -------------------------------------------------------------------------------------------  
+	// -------------------------------------------------------------------------------------------
 	Solver*          solver_;      // my solver
-	DependencyGraph* graph_;       // PBADG
+	GraphPtr         graph_;       // PBADG
 	MiniPtr          mini_;        // minimality checker (only for DLPs)
-	AtomVec          atoms_;       // data for each atom       
+	AtomVec          atoms_;       // data for each atom
 	BodyVec          bodies_;      // data for each body
 	IdQueue          todo_;        // ids of atoms that recently lost their source
 	IdQueue          ufs_;         // ids of atoms that are unfounded wrt the current assignment (limited to one scc)
@@ -275,7 +299,7 @@ private:
 	LitVec           loopAtoms_;   // only used if strategy_ == shared_reason
 	LitVec           activeClause_;// activeClause_[0] is the current unfounded atom
 	LitVec*          reasons_;     // only used if strategy_ == only_reason. reasons_[v] reason why v is unfounded
-	ClauseInfo       info_;        // info on active clause
+	ConstraintInfo   info_;        // info on active clause
 	ReasonStrategy   strategy_;    // what kind of reasons to compute?
 };
 }
