@@ -70,6 +70,7 @@ namespace bk_lib { namespace detail {
 		Fill(const T& val) : val_(val) {}
 		void operator()(T* first, std::size_t n) const { detail::fill(first, first + n, val_); }
 		const T& val_;
+	private: Fill& operator=(const Fill&);
 	};
 	template <class Iter>
 	struct Copy {
@@ -243,15 +244,15 @@ public:
 		return ebo_.buf[n];
 	}
 	
-	//! same as operator[] but throws std::range_error if pre-condition is not met.
+	//! same as operator[] but throws std::out_of_range if pre-condition is not met.
 	const_reference at(size_type n) const {
 		if (n < size()) return ebo_.buf[n];
-		throw std::range_error("pod_vector::at");
+		throw std::out_of_range("pod_vector::at");
 	}
-	//! same as operator[] but throws std::range_error if pre-condition is not met.
+	//! same as operator[] but throws std::out_of_range if pre-condition is not met.
 	reference at(size_type n) {
 		if (n < size()) return ebo_.buf[n];
-		throw std::range_error("pod_vector::at");
+		throw std::out_of_range("pod_vector::at");
 	}
 	
 	//! equivalent to *begin()
