@@ -1,18 +1,18 @@
-// 
+//
 // Copyright (c) 2006-2011, Benjamin Kaufmann
-// 
-// This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
-// 
+//
+// This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/
+//
 // Clasp is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 2 of the License, or
 // (at your option) any later version.
-// 
+//
 // Clasp is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU General Public License
 // along with Clasp; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
@@ -21,7 +21,6 @@
 #define CLASP_SMODELS_CONSTRAINTS_H_INCLUDED
 
 #ifdef _MSC_VER
-#pragma warning (disable : 4200) // nonstandard extension used : zero-sized array
 #pragma once
 #endif
 
@@ -33,10 +32,10 @@ namespace Clasp {
 struct WeightLitsRep {
 	//! Transforms the given literals to the normal form expected by WeightConstraint.
 	/*!
-	 * The function simplifies lits and bound by removing assigned and 
-	 * merging duplicate/complementary literals. Furthermore, negative weights and 
+	 * The function simplifies lits and bound by removing assigned and
+	 * merging duplicate/complementary literals. Furthermore, negative weights and
 	 * their literals are inverted, bound is updated accordingly, and literals
-	 * are sorted by decreasing weight. 
+	 * are sorted by decreasing weight.
 	 */
 	static WeightLitsRep create(Solver& s, WeightLitVec& lits, weight_t bound);
 	//! Propagates the constraint W == *this.
@@ -61,32 +60,37 @@ struct WeightLitsRep {
  * \ingroup constraint
  * This class represents a constraint of type W == w1*x1 ... wn*xn >= B,
  * where W and each xi are literals and B and each wi are strictly positive integers.
+ *
  * The class is used to represent smodels-like weight constraint, i.e.
  * the body of a basic weight rule. In this case W is the literal associated with the body.
  * A cardinality constraint is handled like a weight constraint where all weights are equal to 1.
  *
- * The class implements the following four inference rules:
- * Let L be the set of literals of the constraint,
- * let sumTrue be the sum of the weights of all literals l in L that are currently true,
- * let sumReach be the sum of the weights of all literals l in L that are currently not false,
- * let U = {l in L | value(l.var()) == value_free}
- * - FTB: If sumTrue >= bound: set W to true.
- * - BFB: If W is false: set false all literals l in U for which sumTrue + weight(l) >= bound.
- * - FFB: If sumReach < bound: set W to false.
- * - BTB: If W is true: set true all literals l in U for which sumReach - weight(l) < bound.
+ * <b>Given a WeightConstraint with bound \p B and set of literals \p L</b>,
+ * -  let \p sumTrue be the sum of the weights of all literals \p l in \p L that are currently true,
+ * - \p sumReach be the sum of the weights of all literals \p l in \p L that are currently not false, and
+ * - \p U be the set of literals \p l in \p L that are currently unassigned
+ * .
+ * <b>the class implements the following four inference rules</b>:
+ * - \b FTB: If \p sumTrue >= \p B: assign \p W to true.
+ * - \b BFB: If \p W is false: set false all literals \p l in \p U for which \p sumTrue + weight(\p l) >= \p B.
+ * - \b FFB: If \p sumReach < \p B: assign \p W to false.
+ * - \b BTB: If \p W is true: set true all literals \p l in \p U for which \p sumReach - weight(\p l) < \p B.
+ * .
  */
 class WeightConstraint : public Constraint {
 public:
+	//! Flags controlling weight constraint creation.
 	enum CreationFlags {
-		create_explicit  = 1u, /**< Force creation of explicit constraint even if size/bound is small. */
-		create_no_add    = 3u, /**< Do not add constraint to solver db.    */
-		create_sat       = 4u, /**< Force creation even if constraint is always satisfied.*/
-		create_no_freeze = 8u, /**< Do not freeze variables in constraint. */
-		create_no_share  =16u, /**< Do not allow sharing of literals.      */
-		create_eq_bound  =32u, /**< Create equality instead of less-than constraint. */
-		create_only_btb  =64u, /**< Only create FFB_BTB constraint. */
-		create_only_bfb  =128u,/**< Only create FTB_BFB constraint. */
+		create_explicit  = 1u, //!< Force creation of explicit constraint even if size/bound is small.
+		create_no_add    = 3u, //!< Do not add constraint to solver db.
+		create_sat       = 4u, //!< Force creation even if constraint is always satisfied.
+		create_no_freeze = 8u, //!< Do not freeze variables in constraint.
+		create_no_share  =16u, //!< Do not allow sharing of literals between threads.
+		create_eq_bound  =32u, //!< Create equality instead of greater-or-equal constraint.
+		create_only_btb  =64u, //!< Only create FFB_BTB constraint.
+		create_only_bfb  =128u,//!< Only create FTB_BFB constraint.
 	};
+	//! Type used to communicate result of create().
 	class CPair {
 	public:
 		CPair() { con[0] = con[1] = 0; }
@@ -107,12 +111,19 @@ public:
 	 * \param W The literal that is associated with the constraint.
 	 * \param lits The literals of the weight constraint.
 	 * \param bound The lower bound of the weight constraint.
+	 * \param creationFlags Set of CreationFlags to apply.
 	 * \note Cardinality constraint are represented as weight constraints with all weights equal to 1.
 	 * \note If creationFlags contains create_eq_bound, a constraint W == (lits == bound) is created that
 	 * is represented by up to two weight constraints.
 	 */
 	static CPair create(Solver& s, Literal W, WeightLitVec& lits, weight_t bound, uint32 creationFlags = 0);
-	static CPair create(Solver& s, Literal W, WeightLitsRep rep , uint32 creationFlags);
+
+	//! Low level creation function.
+	/*!
+	 * \note flag create_eq_bound is ignored by this function, that is, this function always creates
+	 * a single >= constraint.
+	 */
+	static CPair create(Solver& s, Literal W, WeightLitsRep& rep, uint32 flags);
 	// constraint interface
 	Constraint* cloneAttach(Solver&);
 	bool simplify(Solver& s, bool = false);
@@ -123,21 +134,22 @@ public:
 	void undoLevel(Solver& s);
 	uint32 estimateComplexity(const Solver& s) const;
 	/*!
-	 * Logically, we distinguish two constraints: 
-	 * FFB_BTB for handling forward false body and backward true body and
-	 * FTB_BFB for handling forward true body and backward false body.
+	 * Logically, we distinguish two constraints:
+	 * - FFB_BTB for handling forward false body and backward true body and
+	 * - FTB_BFB for handling forward true body and backward false body.
+	 * .
 	 * Physically, we store the literals in one array: ~W=1, l0=w0,...,ln-1=wn-1.
 	 */
 	enum ActiveConstraint {
-		FFB_BTB   = 0, /**< (SumW-bound)+1 [~W=1, l0=w0,..., ln-1=wn-1]; */
-		FTB_BFB   = 1, /**< bound          [ W=1,~l0=w0,...,~ln-1=wn-1]  */
+		FFB_BTB   = 0, //!< (\p SumW - \p B)+1 [~W=1, l0=w0,..., ln-1=wn-1]
+		FTB_BFB   = 1, //!< \p B [ W=1,~l0=w0,...,~ln-1=wn-1]
 	};
 	/*!
 	 * Returns the i'th literal of constraint c, i.e.
-	 *  li, iff c == FFB_BTB
-	 * ~li, iff c == FTB_BFB.
+	 * - li, iff c == FFB_BTB
+	 * - ~li, iff c == FTB_BFB.
 	 */
-	Literal  lit(uint32 i, ActiveConstraint c) const { return Literal::fromIndex( lits_->lit(i).index() ^ c ); }
+	Literal  lit(uint32 i, ActiveConstraint c) const { return Literal::fromId( lits_->lit(i).id() ^ c ); }
 	//! Returns the weight of the i'th literal or 1 if constraint is a cardinality constraint.
 	weight_t weight(uint32 i)                  const { return lits_->weight(i); }
 	//! Returns the number of literals in this constraint (including W).
@@ -147,7 +159,7 @@ public:
 	// Returns the index of next literal to look at during backward propagation.
 	uint32   getBpIndex()                      const { return !isWeight() ? 1 : undo_[0].data>>1; }
 private:
-	static WeightConstraint* createImpl(Solver& s, Literal W, WeightLitsRep& rep, uint32 flags);
+	static WeightConstraint* doCreate(Solver& s, Literal W, WeightLitsRep& rep, uint32 flags);
 	bool                     integrateRoot(Solver& s);
 	struct WL {
 		WL(uint32 s, bool shared, bool w);
@@ -157,7 +169,7 @@ private:
 		uint32   size()           const { return sz; }
 		Literal  lit(uint32 i)    const { return lits[(i<<w)]; }
 		Var      var(uint32 i)    const { return lits[(i<<w)].var(); }
-		weight_t weight(uint32 i) const { return !weights() ? weight_t(1) : (weight_t)lits[(i<<1)+1].asUint(); }
+		weight_t weight(uint32 i) const { return !weights() ? weight_t(1) : (weight_t)lits[(i<<1)+1].rep(); }
 		uint32   refCount()       const;
 		WL*      clone();
 		void     release();
@@ -165,14 +177,16 @@ private:
 		uint32  sz : 30; // number of literals
 		uint32  rc : 1;  // ref counted?
 		uint32  w  : 1;  // has weights?
+CLASP_WARNING_BEGIN_RELAXED
 		Literal lits[0]; // Literals of constraint: ~B [Bw], l1 [w1], ..., ln-1 [Wn-1]
+CLASP_WARNING_END_RELAXED
 	};
 	WeightConstraint(Solver& s, SharedContext* ctx, Literal W, const WeightLitsRep& , WL* out, uint32 act = 3u);
 	WeightConstraint(Solver& s, const WeightConstraint& other);
 	~WeightConstraint();
-	
+
 	static const uint32 NOT_ACTIVE = 3u;
-	
+
 	// Represents a literal on the undo stack.
 	// idx()        returns the index of the literal.
 	// constraint() returns the constraint that added the literal to the undo stack.
@@ -182,7 +196,7 @@ private:
 		explicit UndoInfo(uint32 d = 0) : data(d) {}
 		uint32           idx()        const { return data >> 2; }
 		ActiveConstraint constraint() const { return static_cast<ActiveConstraint>((data&2) != 0); }
-		uint32 data; 
+		uint32 data;
 	};
 	// Is literal idx contained as reason lit in the undo stack?
 	bool litSeen(uint32 idx) const { return (undo_[idx].data & 1) != 0; }
@@ -206,7 +220,9 @@ private:
 	uint32   active_ :  2; // which of the two sub-constraints is currently unit?
 	uint32   watched_:  2; // which constraint is watched (3 both, 2 ignore, FTB_BFB, FFB_BTB)
 	weight_t bound_[2];    // FFB_BTB: (sumW-bound)+1 / FTB_BFB: bound
+CLASP_WARNING_BEGIN_RELAXED
 	UndoInfo undo_[0];     // undo stack + seen flag for each literal
+CLASP_WARNING_END_RELAXED
 };
 }
 
