@@ -1,18 +1,18 @@
-// 
-// Copyright (c) 2006-2012, Benjamin Kaufmann
-// 
-// This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
-// 
+//
+// Copyright (c) 2006-2016, Benjamin Kaufmann
+//
+// This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/
+//
 // Clasp is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 2 of the License, or
 // (at your option) any later version.
-// 
+//
 // Clasp is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU General Public License
 // along with Clasp; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
@@ -34,10 +34,10 @@ private:
 	const Solver& solver_;
 };
 
-struct Sink { 
+struct Sink {
 	explicit Sink(SharedLiterals* c) : clause(c) {}
 	~Sink() { if (clause) clause->release(); }
-	SharedLiterals* clause;   
+	SharedLiterals* clause;
 };
 
 void* alloc(uint32 size) {
@@ -58,7 +58,7 @@ SharedLiterals* SharedLiterals::newShareable(const Literal* lits, uint32 size, C
 	return new (m) SharedLiterals(lits, size, t, numRefs);
 }
 
-SharedLiterals::SharedLiterals(const Literal* a_lits, uint32 size, ConstraintType t, uint32 refs) 
+SharedLiterals::SharedLiterals(const Literal* a_lits, uint32 size, ConstraintType t, uint32 refs)
 	: size_type_( (size << 2) + t ) {
 	refCount_ = std::max(uint32(1),refs);
 	std::memcpy(lits_, a_lits, size*sizeof(Literal));
@@ -96,19 +96,18 @@ SharedLiterals* SharedLiterals::share() {
 	++refCount_;
 	return this;
 }
-
 /////////////////////////////////////////////////////////////////////////////////////////
 // ClauseCreator
 /////////////////////////////////////////////////////////////////////////////////////////
-ClauseCreator::ClauseCreator(Solver* s) 
+ClauseCreator::ClauseCreator(Solver* s)
 	: solver_(s)
-	, flags_(0){ 
+	, flags_(0){
 }
 
 ClauseCreator& ClauseCreator::start(ConstraintType t) {
-	assert(solver_ && (solver_->decisionLevel() == 0 || t != Constraint_t::static_constraint));
+	assert(solver_ && (solver_->decisionLevel() == 0 || t != Constraint_t::Static));
 	literals_.clear();
-	extra_ = ClauseInfo(t);
+	extra_ = ConstraintInfo(t);
 	return *this;
 }
 
@@ -122,7 +121,7 @@ uint32 ClauseCreator::watchOrder(const Solver& s, Literal p) {
 	return abstr_p;
 }
 
-ClauseRep ClauseCreator::prepare(Solver& s, const Literal* in, uint32 inSize, const ClauseInfo& e, uint32 flags, Literal* out, uint32 outMax) {
+ClauseRep ClauseCreator::prepare(Solver& s, const Literal* in, uint32 inSize, const ConstraintInfo& e, uint32 flags, Literal* out, uint32 outMax) {
 	assert(out && outMax > 2);
 	ClauseRep ret  = ClauseRep::prepared(out, 0, e);
 	uint32 abst_w1 = 0, abst_w2 = 0;
@@ -147,11 +146,11 @@ ClauseRep ClauseCreator::prepare(Solver& s, const Literal* in, uint32 inSize, co
 			break;
 		}
 	}
-	if (simplify) { 
-		for (uint32 x = 0, end = ret.size; x != end; ++x) { s.clearSeen(out[x].var()); } 
+	if (simplify) {
+		for (uint32 x = 0, end = ret.size; x != end; ++x) { s.clearSeen(out[x].var()); }
 	}
 	if (abst_w1 == UINT32_MAX || (abst_w2 && out[0].var() == out[1].var())) {
-		out[0]   = abst_w1 == UINT32_MAX || out[0] == ~out[1] ? posLit(0) : out[0]; 
+		out[0]   = abst_w1 == UINT32_MAX || out[0] == ~out[1] ? lit_true() : out[0];
 		ret.size = 1;
 	}
 	ret.info.setAux(s.auxVar(vMax));
@@ -159,8 +158,8 @@ ClauseRep ClauseCreator::prepare(Solver& s, const Literal* in, uint32 inSize, co
 }
 
 
-ClauseRep ClauseCreator::prepare(Solver& s, LitVec& lits, uint32 flags, const ClauseInfo& info) {
-	if (lits.empty()) { lits.push_back(negLit(0)); }
+ClauseRep ClauseCreator::prepare(Solver& s, LitVec& lits, uint32 flags, const ConstraintInfo& info) {
+	if (lits.empty()) { lits.push_back(lit_false()); }
 	if ((flags & clause_no_prepare) == 0 || (flags & clause_force_simplify) != 0) {
 		ClauseRep x = prepare(s, &lits[0], (uint32)lits.size(), info, flags, &lits[0]);
 		shrinkVecTo(lits, x.size);
@@ -176,14 +175,14 @@ ClauseRep ClauseCreator::prepare(bool forceSimplify) {
 ClauseCreator::Status ClauseCreator::status(const Solver& s, const Literal* clause_begin, const Literal* clause_end) {
 	if (clause_end <= clause_begin) { return status_empty; }
 	Literal temp[3];
-	ClauseRep x = prepare(const_cast<Solver&>(s), clause_begin, uint32(clause_end - clause_begin), ClauseInfo(), 0, temp, 3);
+	ClauseRep x = prepare(const_cast<Solver&>(s), clause_begin, uint32(clause_end - clause_begin), ConstraintInfo(), 0, temp, 3);
 	return status(s, x);
 }
 
 ClauseCreator::Status ClauseCreator::status(const Solver& s, const ClauseRep& c) {
 	uint32 dl = s.decisionLevel();
 	uint32 fw = c.size     ? watchOrder(s, c.lits[0]) : 0;
-	if (fw == UINT32_MAX) { return status_subsumed; }	
+	if (fw == UINT32_MAX) { return status_subsumed; }
 	uint32 sw = c.size > 1 ? watchOrder(s, c.lits[1]) : 0;
 	uint32 st = status_open;
 	if      (fw > varMax)   { st|= status_sat; fw = ~fw; }
@@ -207,7 +206,7 @@ ClauseCreator::Result ClauseCreator::end(uint32 flags) {
 
 ClauseHead* ClauseCreator::newProblemClause(Solver& s, const ClauseRep& clause, uint32 flags) {
 	ClauseHead* ret;
-	Solver::WatchInitMode wMode = s.watchInitMode();	
+	Solver::WatchInitMode wMode = s.watchInitMode();
 	if      (flags&clause_watch_first){ wMode = SolverStrategies::watch_first;}
 	else if (flags&clause_watch_rand) { wMode = SolverStrategies::watch_rand; }
 	else if (flags&clause_watch_least){ wMode = SolverStrategies::watch_least;}
@@ -215,7 +214,7 @@ ClauseHead* ClauseCreator::newProblemClause(Solver& s, const ClauseRep& clause,
 		uint32 fw = 0, sw = 1;
 		if (wMode == SolverStrategies::watch_rand) {
 			fw = s.rng.irand(clause.size);
-			do { sw = s.rng.irand(clause.size); } while (sw == fw); 
+			do { sw = s.rng.irand(clause.size); } while (sw == fw);
 		}
 		else if (wMode == SolverStrategies::watch_least) {
 			uint32 cw1 = s.numWatches(~clause.lits[0]);
@@ -266,7 +265,7 @@ ClauseHead* ClauseCreator::newLearntClause(Solver& s, const ClauseRep& clause, u
 	return ret;
 }
 
-ClauseHead* ClauseCreator::newUnshared(Solver& s, SharedLiterals* clause, const Literal* w, const ClauseInfo& e) {
+ClauseHead* ClauseCreator::newUnshared(Solver& s, SharedLiterals* clause, const Literal* w, const ConstraintInfo& e) {
 	LitVec temp; temp.reserve(clause->size());
 	temp.assign(w, w+2);
 	for (const Literal* x = clause->begin(), *end = clause->end(); x != end; ++x) {
@@ -280,7 +279,7 @@ ClauseHead* ClauseCreator::newUnshared(Solver& s, SharedLiterals* clause, const
 ClauseCreator::Result ClauseCreator::create_prepared(Solver& s, const ClauseRep& clause, uint32 flags) {
 	assert(s.decisionLevel() == 0 || (clause.info.learnt() && clause.prep));
 	Status x = status(s, clause);
-	if (ignoreClause(s, clause, x, flags)){ 
+	if (ignoreClause(s, clause, x, flags)){
 		return Result(0, x);
 	}
 	if (clause.size > 1) {
@@ -307,12 +306,12 @@ ClauseCreator::Result ClauseCreator::create_prepared(Solver& s, const ClauseRep&
 	return Result(0, !s.hasConflict() ? status_unit : status_unsat);
 }
 
-ClauseCreator::Result ClauseCreator::create(Solver& s, LitVec& lits, uint32 flags, const ClauseInfo& extra) {
+ClauseCreator::Result ClauseCreator::create(Solver& s, LitVec& lits, uint32 flags, const ConstraintInfo& extra) {
 	return create_prepared(s, prepare(s, lits, flags, extra), flags);
 }
 
 ClauseCreator::Result ClauseCreator::create(Solver& s, const ClauseRep& rep, uint32 flags) {
-	return create_prepared(s, (rep.prep == 0 && (flags & clause_no_prepare) == 0 
+	return create_prepared(s, (rep.prep == 0 && (flags & clause_no_prepare) == 0
 		? prepare(s, rep.lits, rep.size, rep.info, flags, rep.lits)
 		: ClauseRep::prepared(rep.lits, rep.size, rep.info)), flags);
 }
@@ -321,8 +320,8 @@ ClauseCreator::Result ClauseCreator::integrate(Solver& s, SharedLiterals* clause
 	assert(!s.hasConflict() && "ClauseCreator::integrate() - precondition violated!");
 	Detail::Sink shared( (modeFlags & clause_no_release) == 0 ? clause : 0);
 	// determine state of clause
-	Literal temp[Clause::MAX_SHORT_LEN]; temp[0] = temp[1] = negLit(0);
-	ClauseRep x    = prepare(s, clause->begin(), clause->size(), ClauseInfo(t), 0, temp, Clause::MAX_SHORT_LEN);
+	Literal temp[Clause::MAX_SHORT_LEN]; temp[0] = temp[1] = lit_false();
+	ClauseRep x    = prepare(s, clause->begin(), clause->size(), ConstraintInfo(t), 0, temp, Clause::MAX_SHORT_LEN);
 	uint32 impSize = (modeFlags & clause_explicit) != 0 || !s.allowImplicit(x) ? 1 : 3;
 	Status xs      = status(s, x);
 	if (ignoreClause(s, x, xs, modeFlags)) {
@@ -342,20 +341,21 @@ ClauseCreator::Result ClauseCreator::integrate(Solver& s, SharedLiterals* clause
 		// only check for implication/conflict but do not create
 		// a local representation for the clause
 		s.stats.addLearnt(x.size, x.info.type());
-		modeFlags    |= clause_no_add;
+		modeFlags |= clause_no_add;
 	}
 	if ((modeFlags & clause_no_add) == 0) { s.addLearnt(result.local, x.size, x.info.type()); }
 	if ((xs & (status_unit|status_unsat)) != 0) {
 		Antecedent ante = result.local ? Antecedent(result.local) : Antecedent(~temp[1], ~temp[2]);
 		uint32 impLevel = s.level(temp[1].var());
 		result.status   = s.force(temp[0], impLevel, ante) ? status_unit : status_unsat;
-		if (result.local && (modeFlags & clause_int_lbd) != 0 && result.status != status_unsat) {
-			result.local->lbd(s.updateLearnt(negLit(0), clause->begin(), clause->end(), result.local->lbd(), true));
+		if (result.local && (modeFlags & clause_int_lbd) != 0) {
+			uint32 lbd = s.countLevels(clause->begin(), clause->end());
+			result.local->resetScore(makeScore(x.info.activity(), lbd));
 		}
 	}
 	return result;
 }
-ClauseCreator::Result ClauseCreator::integrate(Solver& s, SharedLiterals* clause, uint32 modeFlags) { 
+ClauseCreator::Result ClauseCreator::integrate(Solver& s, SharedLiterals* clause, uint32 modeFlags) {
 	return integrate(s, clause, modeFlags, clause->type());
 }
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -366,7 +366,7 @@ void* Clause::alloc(Solver& s, uint32 lits, bool learnt) {
 		if (learnt) { s.addLearntBytes(32); }
 		return s.allocSmall();
 	}
-	uint32 extra = std::max((uint32)ClauseHead::HEAD_LITS, lits) - ClauseHead::HEAD_LITS; 
+	uint32 extra = std::max((uint32)ClauseHead::HEAD_LITS, lits) - ClauseHead::HEAD_LITS;
 	uint32 bytes = sizeof(Clause) + (extra)*sizeof(Literal);
 	if (learnt) { s.addLearntBytes(bytes); }
 	return Detail::alloc(bytes);
@@ -377,7 +377,7 @@ ClauseHead* Clause::newClause(void* mem, Solver& s, const ClauseRep& rep) {
 	return new (mem) Clause(s, rep);
 }
 
-ClauseHead* Clause::newShared(Solver& s, SharedLiterals* shared_lits, const ClauseInfo& e, const Literal* lits, bool addRef) {
+ClauseHead* Clause::newShared(Solver& s, SharedLiterals* shared_lits, const InfoType& e, const Literal* lits, bool addRef) {
 	return mt::SharedLitsClause::newClause(s, shared_lits, e, lits, addRef);
 }
 
@@ -386,45 +386,48 @@ ClauseHead* Clause::newContractedClause(Solver& s, const ClauseRep& rep, uint32
 	if (extend) { std::stable_sort(rep.lits+tailStart, rep.lits+rep.size, Detail::GreaterLevel(s)); }
 	return new (alloc(s, rep.size, rep.info.learnt())) Clause(s, rep, tailStart, extend);
 }
-
-Clause::Clause(Solver& s, const ClauseRep& rep, uint32 tail, bool extend) 
+void ClauseHead::Local::init(uint32 sz) {
+	std::memset(mem, 0, sizeof(mem));
+	if (sz > ClauseHead::MAX_SHORT_LEN) { mem[0] = (sz << 3) + 1; }
+	assert(isSmall() == (sz <= ClauseHead::MAX_SHORT_LEN));
+}
+Clause::Clause(Solver& s, const ClauseRep& rep, uint32 tail, bool extend)
 	: ClauseHead(rep.info) {
 	assert(tail >= rep.size || s.isFalse(rep.lits[tail]));
-	data_.local.init(rep.size);
+	local_.init(rep.size);
 	if (!isSmall()) {
 		// copy literals
 		std::memcpy(head_, rep.lits, rep.size*sizeof(Literal));
 		tail = std::max(tail, (uint32)ClauseHead::HEAD_LITS);
 		if (tail < rep.size) {       // contracted clause
-			head_[rep.size-1].watch(); // mark last literal of clause
+			head_[rep.size-1].flag();  // mark last literal of clause
 			Literal t = head_[tail];
 			if (s.level(t.var()) > 0) {
-				data_.local.markContracted();
+				local_.markContracted();
 				if (extend) {
 					s.addUndoWatch(s.level(t.var()), this);
 				}
 			}
-			data_.local.setSize(tail);
+			local_.setSize(tail);
 		}
 	}
 	else {
 		std::memcpy(head_, rep.lits, std::min(rep.size, (uint32)ClauseHead::HEAD_LITS)*sizeof(Literal));
-		data_.lits[0] = rep.size > ClauseHead::HEAD_LITS   ? rep.lits[ClauseHead::HEAD_LITS].asUint()   : negLit(0).asUint();
-		data_.lits[1] = rep.size > ClauseHead::HEAD_LITS+1 ? rep.lits[ClauseHead::HEAD_LITS+1].asUint() : negLit(0).asUint();
+		small()[0] = rep.size > ClauseHead::HEAD_LITS   ? rep.lits[ClauseHead::HEAD_LITS]   : lit_false();
+		small()[1] = rep.size > ClauseHead::HEAD_LITS+1 ? rep.lits[ClauseHead::HEAD_LITS+1] : lit_false();
 		assert(isSmall() && Clause::size() == rep.size);
 	}
 	attach(s);
 }
 
-Clause::Clause(Solver& s, const Clause& other) : ClauseHead(ClauseInfo()) {
-	info_.rep      = other.info_.rep;
-	uint32 oSize   = other.size();
-	data_.local.init(oSize);
+Clause::Clause(Solver& s, const Clause& other) : ClauseHead(other.info_) {
+	uint32 oSize = other.size();
+	local_.init(oSize);
 	if      (!isSmall())      { std::memcpy(head_, other.head_, oSize*sizeof(Literal)); }
-	else if (other.isSmall()) { std::memcpy(data_.lits, other.data_.lits, (ClauseHead::MAX_SHORT_LEN+1)*sizeof(Literal)); }
+	else if (other.isSmall()) { std::memcpy(&local_, &other.local_, (ClauseHead::MAX_SHORT_LEN+1)*sizeof(Literal)); }
 	else { // this is small, other is not
 		std::memcpy(head_, other.head_, ClauseHead::HEAD_LITS*sizeof(Literal));
-		std::memcpy(data_.lits, other.head_+ClauseHead::HEAD_LITS, 2*sizeof(Literal));
+		std::memcpy(&local_, other.head_+ClauseHead::HEAD_LITS, 2*sizeof(Literal));
 	}
 	attach(s);
 }
@@ -433,9 +436,12 @@ Constraint* Clause::cloneAttach(Solver& other) {
 	assert(!learnt());
 	return new (alloc(other, Clause::size(), false)) Clause(other, *this);
 }
-
+Literal* Clause::small()          { return static_cast<Literal*>(static_cast<void*>(&local_)); }
+bool Clause::contracted()   const { return local_.contracted(); }
+bool Clause::isSmall()      const { return local_.isSmall(); }
+bool Clause::strengthened() const { return local_.strengthened(); }
 void Clause::destroy(Solver* s, bool detachFirst) {
-	if (s) { 
+	if (s) {
 		if (detachFirst) { Clause::detach(*s); }
 		if (learnt())    { s->freeLearntBytes(computeAllocSize()); }
 	}
@@ -448,7 +454,7 @@ void Clause::destroy(Solver* s, bool detachFirst) {
 
 void Clause::detach(Solver& s) {
 	if (contracted()) {
-		Literal* eoc = longEnd();
+		Literal* eoc = end();
 		if (s.isFalse(*eoc) && s.level(eoc->var()) != 0) {
 			s.removeUndoWatch(s.level(eoc->var()), this);
 		}
@@ -459,41 +465,46 @@ void Clause::detach(Solver& s) {
 uint32 Clause::computeAllocSize() const {
 	if (isSmall()) { return 32; }
 	uint32 rt = sizeof(Clause) - (ClauseHead::HEAD_LITS*sizeof(Literal));
-	uint32 sz = data_.local.size();
+	uint32 sz = local_.size();
 	uint32 nw = contracted() + strengthened();
 	if (nw != 0u) {
 		const Literal* eoc = head_ + sz;
-		do { nw -= eoc++->watched(); } while (nw); 
+		do { nw -= eoc++->flagged(); } while (nw);
 		sz = static_cast<uint32>(eoc - head_);
 	}
 	return rt + (sz*sizeof(Literal));
 }
 
 bool Clause::updateWatch(Solver& s, uint32 pos) {
-	uint32* n;
+	Literal* it;
 	if (!isSmall()) {
-		for (Literal* begin = head_ + ClauseHead::HEAD_LITS, *end = longEnd(), *first = begin + data_.local.idx;;) {
-			for (Literal* it  = first; it < end; ++it) {
+		for (Literal* begin = head_ + ClauseHead::HEAD_LITS, *end = this->end(), *first = begin + local_.mem[1];;) {
+			for (it = first; it < end; ++it) {
 				if (!s.isFalse(*it)) {
 					std::swap(*it, head_[pos]);
-					data_.local.idx = static_cast<uint32>(++it - begin);
+					local_.mem[1] = static_cast<uint32>(++it - begin);
 					return true;
 				}
 			}
 			if (first == begin) { break; }
-			end   = first;
+			end = first;
 			first = begin;
 		}
 	}
-	else if (!s.isFalse(Literal::fromRep(*(n = data_.lits))) || !s.isFalse(Literal::fromRep(*++n))) {
-		std::swap(head_[pos].asUint(), *n);
+	else if (!s.isFalse(*(it = this->small())) || !s.isFalse(*++it)) {
+		std::swap(head_[pos], *it);
 		return true;
 	}
 	return false;
 }
-
+Clause::LitRange Clause::tail() {
+	if (!isSmall()) { return LitRange(head_+ClauseHead::HEAD_LITS, end()); }
+	Literal* tBeg = small(), *tEnd = tBeg;
+	if (*tEnd != lit_false()) ++tEnd;
+	if (*tEnd != lit_false()) ++tEnd;
+	return LitRange(tBeg, tEnd);
+}
 void Clause::reason(Solver& s, Literal p, LitVec& out) {
-	LitVec::size_type i = out.size();
 	out.push_back(~head_[p == head_[0]]);
 	if (!isSentinel(head_[2])) {
 		out.push_back(~head_[2]);
@@ -503,17 +514,16 @@ void Clause::reason(Solver& s, Literal p, LitVec& out) {
 		}
 		if (contracted()) {
 			const Literal* r = t.second;
-			do { out.push_back(~*r); } while (!r++->watched());
+			do { out.push_back(~*r); } while (!r++->flagged());
 		}
 	}
-	if (learnt()) { 
-		ClauseHead::bumpActivity();
-		setLbd(s.updateLearnt(p, &out[0]+i, &out[0]+out.size(), lbd(), !hasLbd())); 
+	if (learnt()) {
+		s.updateOnReason(info_.score(), p, out);
 	}
 }
 
 bool Clause::minimize(Solver& s, Literal p, CCMinRecursive* rec) {
-	ClauseHead::bumpActivity();
+	s.updateOnMinimize(info_.score());
 	uint32 other = p == head_[0];
 	if (!s.ccMinimize(~head_[other], rec) || !s.ccMinimize(~head_[2], rec)) { return false; }
 	LitRange t = tail();
@@ -523,7 +533,7 @@ bool Clause::minimize(Solver& s, Literal p, CCMinRecursive* rec) {
 	if (contracted()) {
 		do {
 			if (!s.ccMinimize(~*t.second, rec)) { return false; }
-		} while (!t.second++->watched());
+		} while (!t.second++->flagged());
 	}
 	return true;
 }
@@ -539,7 +549,7 @@ bool Clause::isReverseReason(const Solver& s, Literal p, uint32 maxL, uint32 max
 	}
 	if (contracted()) {
 		const Literal* r = t.second;
-		do { notSeen += !s.seen(r->var()); } while (notSeen <= maxN && !r++->watched());
+		do { notSeen += !s.seen(r->var()); } while (notSeen <= maxN && !r++->flagged());
 	}
 	return notSeen <= maxN;
 }
@@ -560,26 +570,26 @@ bool Clause::simplify(Solver& s, bool reinit) {
 		else if (s.isTrue(*it)) { Clause::detach(s); return true;}
 	}
 	// replace any false lits with sentinels
-	for (Literal* r = j; r != t.second; ++r) { *r = negLit(0); }
+	for (Literal* r = j; r != t.second; ++r) { *r = lit_false(); }
 	if (!isSmall()) {
-		uint32 size     = std::max((uint32)ClauseHead::HEAD_LITS, static_cast<uint32>(j-head_));
-		data_.local.idx = 0;
-		data_.local.setSize(size);
+		uint32 size = std::max((uint32)ClauseHead::HEAD_LITS, static_cast<uint32>(j-head_));
+		local_.setSize(size);
+		local_.clearIdx();
 		if (j != t.second && learnt() && !strengthened()) {
 			// mark last literal so that we can recompute alloc size later
-			t.second[-1].watch();
-			data_.local.markStrengthened();
+			t.second[-1].flag();
+			local_.markStrengthened();
 		}
 		if (reinit && size > 3) {
 			detach(s);
 			std::random_shuffle(head_, j, s.rng);
-			attach(s);	
+			attach(s);
 		}
 	}
 	else if (s.isFalse(head_[2])) {
 		head_[2]   = t.first[0];
 		t.first[0] = t.first[1];
-		t.first[1] = negLit(0);
+		t.first[1] = lit_false();
 		--j;
 	}
 	return j <= t.first && ClauseHead::toImplication(s);
@@ -609,27 +619,28 @@ uint32 Clause::isOpen(const Solver& s, const TypeSet& x, LitVec& freeLits) {
 
 void Clause::undoLevel(Solver& s) {
 	assert(!isSmall());
-	uint32   t = data_.local.size();
+	uint32   t = local_.size();
+	uint32  ul = s.jumpLevel();
 	Literal* r = head_+t;
-	while (!r->watched() && s.value(r->var()) == value_free) {
+	while (!r->flagged() && (s.value(r->var()) == value_free || s.level(r->var()) > ul)) {
 		++t;
 		++r;
 	}
-	if (r->watched() || s.level(r->var()) == 0) {
-		r->clearWatch();
+	if (r->flagged() || s.level(r->var()) == 0) {
+		r->unflag();
 		t += !isSentinel(*r);
-		data_.local.clearContracted();
+		local_.clearContracted();
 	}
 	else {
 		s.addUndoWatch(s.level(r->var()), this);
 	}
-	data_.local.setSize(t);
+	local_.setSize(t);
 }
 
 void Clause::toLits(LitVec& out) const {
 	out.insert(out.end(), head_, (head_+ClauseHead::HEAD_LITS)-isSentinel(head_[2]));
 	LitRange t = const_cast<Clause&>(*this).tail();
-	if (contracted()) { while (!t.second++->watched()) {;} }
+	if (contracted()) { while (!t.second++->flagged()) {;} }
 	out.insert(out.end(), t.first, t.second);
 }
 
@@ -645,16 +656,16 @@ ClauseHead::BoolPair Clause::strengthen(Solver& s, Literal p, bool toShort) {
 			s.removeWatch(~p, this);
 			Literal* best = it, *n;
 			for (n = t.first; n != eot && s.isFalse(*best); ++n) {
-				if (!s.isFalse(*n) || s.level(n->var()) > s.level(best->var())) { 
-					best = n; 
+				if (!s.isFalse(*n) || s.level(n->var()) > s.level(best->var())) {
+					best = n;
 				}
 			}
 			std::swap(*it, *best);
 			s.addWatch(~*it, ClauseWatch(this));
 			it = head_+2;
-		}	
+		}
 		// replace cache literal with literal from tail
-		if ((*it  = *t.first) != negLit(0)) {
+		if ((*it  = *t.first) != lit_false()) {
 			eot     = removeFromTail(s, t.first, eot);
 		}
 		ret.first = true;
@@ -664,7 +675,7 @@ ClauseHead::BoolPair Clause::strengthen(Solver& s, Literal p, bool toShort) {
 		ret.first = true;
 	}
 	else if (contracted()) {
-		for (; *it != p && !it->watched(); ++it) { ; }
+		for (; *it != p && !it->flagged(); ++it) { ; }
 		ret.first = *it == p;
 		eot       = *it == p ? removeFromTail(s, it, eot) : it + 1;
 	}
@@ -679,28 +690,28 @@ Literal* Clause::removeFromTail(Solver& s, Literal* it, Literal* end) {
 	assert(it != end || contracted());
 	if (!contracted()) {
 		*it  = *--end;
-		*end = negLit(0);
-		if (!isSmall()) { 
-			data_.local.setSize(data_.local.size()-1);
-			data_.local.idx = 0;
+		*end = lit_false();
+		if (!isSmall()) {
+			local_.setSize(local_.size()-1);
+			local_.clearIdx();
 		}
 	}
 	else {
 		uint32 uLev  = s.level(end->var());
 		Literal* j   = it;
-		while ( !j->watched() ) { *j++ = *++it; }
-		*j           = negLit(0);
+		while ( !j->flagged() ) { *j++ = *++it; }
+		*j           = lit_false();
 		uint32 nLev  = s.level(end->var());
 		if (uLev != nLev && s.removeUndoWatch(uLev, this) && nLev != 0) {
 			s.addUndoWatch(nLev, this);
 		}
-		if (j != end) { (j-1)->watch(); }
-		else          { data_.local.clearContracted(); }
+		if (j != end) { (j-1)->flag(); }
+		else          { local_.clearContracted(); }
 		end = j;
 	}
 	if (learnt() && !isSmall() && !strengthened()) {
-		end->watch();
-		data_.local.markStrengthened();
+		end->flag();
+		local_.markStrengthened();
 	}
 	return end;
 }
@@ -714,27 +725,27 @@ uint32 Clause::size() const {
 // mt::SharedLitsClause
 /////////////////////////////////////////////////////////////////////////////////////////
 namespace mt {
-ClauseHead* SharedLitsClause::newClause(Solver& s, SharedLiterals* shared_lits, const ClauseInfo& e, const Literal* lits, bool addRef) {
+ClauseHead* SharedLitsClause::newClause(Solver& s, SharedLiterals* shared_lits, const InfoType& e, const Literal* lits, bool addRef) {
 	return new (s.allocSmall()) SharedLitsClause(s, shared_lits, lits, e, addRef);
 }
 
-SharedLitsClause::SharedLitsClause(Solver& s, SharedLiterals* shared_lits, const Literal* w, const ClauseInfo& e, bool addRef) 
+SharedLitsClause::SharedLitsClause(Solver& s, SharedLiterals* shared_lits, const Literal* w, const InfoType& e, bool addRef)
 	: ClauseHead(e) {
 	static_assert(sizeof(SharedLitsClause) <= 32, "Unsupported Padding");
-	data_.shared = addRef ? shared_lits->share() : shared_lits;
+	shared_ = addRef ? shared_lits->share() : shared_lits;
 	std::memcpy(head_, w, std::min((uint32)ClauseHead::HEAD_LITS, shared_lits->size())*sizeof(Literal));
 	attach(s);
 	if (learnt()) { s.addLearntBytes(32); }
 }
 
 Constraint* SharedLitsClause::cloneAttach(Solver& other) {
-	return SharedLitsClause::newClause(other, data_.shared, ClauseInfo(this->type()), head_);
+	return SharedLitsClause::newClause(other, shared_, InfoType(this->type()), head_);
 }
 
 bool SharedLitsClause::updateWatch(Solver& s, uint32 pos) {
 	Literal  other = head_[1^pos];
-	for (const Literal* r = data_.shared->begin(), *end = data_.shared->end(); r != end; ++r) {
-		// at this point we know that head_[2] is false so we only need to check 
+	for (const Literal* r = shared_->begin(), *end = shared_->end(); r != end; ++r) {
+		// at this point we know that head_[2] is false so we only need to check
 		// that we do not watch the other watched literal twice!
 		if (!s.isFalse(*r) && *r != other) {
 			head_[pos] = *r; // replace watch
@@ -755,20 +766,18 @@ bool SharedLitsClause::updateWatch(Solver& s, uint32 pos) {
 }
 
 void SharedLitsClause::reason(Solver& s, Literal p, LitVec& out) {
-	LitVec::size_type i = out.size();
-	for (const Literal* r = data_.shared->begin(), *end = data_.shared->end(); r != end; ++r) {
+	for (const Literal* r = shared_->begin(), *end = shared_->end(); r != end; ++r) {
 		assert(s.isFalse(*r) || *r == p);
 		if (*r != p) { out.push_back(~*r); }
 	}
-	if (learnt()) { 
-		ClauseHead::bumpActivity();
-		setLbd(s.updateLearnt(p, &out[0]+i, &out[0]+out.size(), lbd(), !hasLbd())); 
+	if (learnt()) {
+		s.updateOnReason(info_.score(), p, out);
 	}
 }
 
 bool SharedLitsClause::minimize(Solver& s, Literal p, CCMinRecursive* rec) {
-	ClauseHead::bumpActivity();
-	for (const Literal* r = data_.shared->begin(), *end = data_.shared->end(); r != end; ++r) {
+	s.updateOnMinimize(info_.score());
+	for (const Literal* r = shared_->begin(), *end = shared_->end(); r != end; ++r) {
 		if (*r != p && !s.ccMinimize(~*r, rec)) { return false; }
 	}
 	return true;
@@ -776,7 +785,7 @@ bool SharedLitsClause::minimize(Solver& s, Literal p, CCMinRecursive* rec) {
 
 bool SharedLitsClause::isReverseReason(const Solver& s, Literal p, uint32 maxL, uint32 maxN) {
 	uint32 notSeen = 0;
-	for (const Literal* r = data_.shared->begin(), *end = data_.shared->end(); r != end; ++r) {
+	for (const Literal* r = shared_->begin(), *end = shared_->end(); r != end; ++r) {
 		if (*r == p) continue;
 		if (!isRevLit(s, *r, maxL)) return false;
 		if (!s.seen(r->var()) && ++notSeen > maxN) return false;
@@ -789,7 +798,7 @@ bool SharedLitsClause::simplify(Solver& s, bool reinit) {
 		detach(s);
 		return true;
 	}
-	uint32 optSize = data_.shared->simplify(s);
+	uint32 optSize = shared_->simplify(s);
 	if (optSize == 0) {
 		detach(s);
 		return true;
@@ -797,22 +806,21 @@ bool SharedLitsClause::simplify(Solver& s, bool reinit) {
 	else if (optSize <= Clause::MAX_SHORT_LEN) {
 		Literal lits[Clause::MAX_SHORT_LEN];
 		Literal* j = lits;
-		for (const Literal* r = data_.shared->begin(), *e = data_.shared->end(); r != e; ++r) {
+		for (const Literal* r = shared_->begin(), *e = shared_->end(); r != e; ++r) {
 			if (!s.isFalse(*r)) *j++ = *r;
 		}
-		uint32 rep= info_.rep;
+		// safe extra data
+		InfoType myInfo = info_;
 		// detach & destroy but do not release memory
 		detach(s);
 		SharedLitsClause::destroy(0, false);
 		// construct short clause in "this"
-		ClauseHead* h = Clause::newClause(this, s, ClauseRep::prepared(lits, static_cast<uint32>(j-lits)));
-		// restore extra data - safe because memory layout has not changed!
-		info_.rep = rep;
+		ClauseHead* h = Clause::newClause(this, s, ClauseRep::prepared(lits, static_cast<uint32>(j-lits), myInfo));
 		return h->simplify(s, reinit);
 	}
 	else if (s.isFalse(head_[2])) {
 		// try to replace cache lit with non-false literal
-		for (const Literal* r = data_.shared->begin(), *e = data_.shared->end(); r != e; ++r) {
+		for (const Literal* r = shared_->begin(), *e = shared_->end(); r != e; ++r) {
 			if (!s.isFalse(*r) && std::find(head_, head_+2, *r) == head_+2) {
 				head_[2] = *r;
 				break;
@@ -827,7 +835,7 @@ void SharedLitsClause::destroy(Solver* s, bool detachFirst) {
 		if (detachFirst) { ClauseHead::detach(*s); }
 		if (learnt())    { s->freeLearntBytes(32); }
 	}
-	data_.shared->release();
+	shared_->release();
 	void* mem = this;
 	this->~SharedLitsClause();
 	if (s) { s->freeSmall(mem); }
@@ -839,7 +847,7 @@ uint32 SharedLitsClause::isOpen(const Solver& s, const TypeSet& x, LitVec& freeL
 	}
 	Literal* head = head_;
 	ValueRep v;
-	for (const Literal* r = data_.shared->begin(), *end = data_.shared->end(); r != end; ++r) {
+	for (const Literal* r = shared_->begin(), *end = shared_->end(); r != end; ++r) {
 		if ( (v = s.value(r->var())) == value_free ) {
 			freeLits.push_back(*r);
 		}
@@ -852,15 +860,14 @@ uint32 SharedLitsClause::isOpen(const Solver& s, const TypeSet& x, LitVec& freeL
 }
 
 void SharedLitsClause::toLits(LitVec& out) const {
-	out.insert(out.end(), data_.shared->begin(), data_.shared->end());
+	out.insert(out.end(), shared_->begin(), shared_->end());
 }
 
 ClauseHead::BoolPair SharedLitsClause::strengthen(Solver&, Literal, bool) {
 	return BoolPair(false, false);
 }
 
-uint32 SharedLitsClause::size() const { return data_.shared->size(); }
-
+uint32 SharedLitsClause::size() const { return shared_->size(); }
 } // end namespace mt
 
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -872,31 +879,31 @@ LoopFormula* LoopFormula::newLoopFormula(Solver& s, const ClauseRep& c1, const L
 	s.addLearntBytes(bytes);
 	return new (mem)LoopFormula(s, c1, atoms, nAtoms, heu);
 }
-LoopFormula::LoopFormula(Solver& s, const ClauseRep& c1, const Literal* atoms, uint32 nAtoms, bool heu)
-	: act_(c1.info.activity(), c1.info.lbd()) {
-	lits_[0] = posLit(0); // Starting sentinel
+LoopFormula::LoopFormula(Solver& s, const ClauseRep& c1, const Literal* atoms, uint32 nAtoms, bool heu) {
+	act_ = c1.info.score();
+	lits_[0] = lit_true(); // Starting sentinel
 	std::memcpy(lits_ + 1, c1.lits, c1.size * sizeof(Literal));
-	lits_[end_ = c1.size + 1] = posLit(0); // Ending sentinel
+	lits_[end_ = c1.size + 1] = lit_true(); // Ending sentinel
 	s.addWatch(~lits_[2], this, (2 << 1) + 1);
-	lits_[2].watch();
+	lits_[2].flag();
 	size_  = c1.size + nAtoms + 2;
 	str_   = 0;
 	xPos_  = 1;
 	other_ = 1;
 	for (uint32 i = 0, x = end_ + 1; i != nAtoms; ++i, ++x) {
-		act_.bumpAct();
+		act_.bumpActivity();
 		s.addWatch(~(lits_[x] = atoms[i]), this, (1 << 1) + 1);
 		if (heu) {
 			lits_[1] = atoms[i];
-			s.heuristic()->newConstraint(s, lits_ + 1, c1.size, Constraint_t::learnt_loop);
+			s.heuristic()->newConstraint(s, lits_ + 1, c1.size, Constraint_t::Loop);
 		}
 	}
-	(lits_[1] = c1.lits[0]).watch();
+	(lits_[1] = c1.lits[0]).flag();
 }
 void LoopFormula::destroy(Solver* s, bool detach) {
 	if (s) {
 		if (detach) { this->detach(*s); }
-		if (str_)   { while (lits_[size_++].asUint() != 3u) { ; } }	
+		if (str_)   { while (lits_[size_++].rep() != 3u) { ; } }
 		s->freeLearntBytes(sizeof(LoopFormula) + (size_ * sizeof(Literal)));
 	}
 	void* mem = static_cast<Constraint*>(this);
@@ -905,7 +912,7 @@ void LoopFormula::destroy(Solver* s, bool detach) {
 }
 void LoopFormula::detach(Solver& s) {
 	for (Literal* it = begin() + xPos_; !isSentinel(*it); ++it) {
-		if (it->watched()) { s.removeWatch(~*it, this); it->clearWatch(); }
+		if (it->flagged()) { s.removeWatch(~*it, this); it->unflag(); }
 	}
 	for (Literal* it = xBegin(), *end = xEnd(); it != end; ++it) {
 		s.removeWatch(~*it, this);
@@ -916,7 +923,7 @@ bool LoopFormula::otherIsSat(const Solver& s) {
 	if (!s.isTrue(lits_[other_])) { return false; }
 	for (Literal* it = xBegin(), *end = xEnd(); it != end; ++it) {
 		if (!s.isTrue(*it)) {
-			if (lits_[xPos_].watched()){ (lits_[xPos_] = *it).watch(); }
+			if (lits_[xPos_].flagged()){ (lits_[xPos_] = *it).flag(); }
 			else                       { lits_[xPos_] = *it; }
 			return false;
 		}
@@ -925,7 +932,7 @@ bool LoopFormula::otherIsSat(const Solver& s) {
 }
 Constraint::PropResult LoopFormula::propagate(Solver& s, Literal p, uint32& data) {
 	if (otherIsSat(s)) { // already satisfied?
-		return PropResult(true, true); 
+		return PropResult(true, true);
 	}
 	uint32 idx = data >> 1;
 	Literal* w = lits_ + idx;
@@ -933,19 +940,19 @@ Constraint::PropResult LoopFormula::propagate(Solver& s, Literal p, uint32& data
 	if (head) { // p is one of the atoms - move to active part
 		p = ~p;
 		if (*w != p && s.isFalse(*w)) { return PropResult(true, true); }
-		if (!w->watched())            { *w = p; return PropResult(true, true); }
-		(*w = p).watch();
+		if (!w->flagged())            { *w = p; return PropResult(true, true); }
+		(*w = p).flag();
 	}
 	for (int bounds = 0, dir = ((data & 1) << 1) - 1;;) {
 		// search non-false literal - sentinels guarantee termination
-		for (w += dir; s.isFalse(*w); w += dir) {;} 
+		for (w += dir; s.isFalse(*w); w += dir) {;}
 		if (!isSentinel(*w)) {
 			uint32 nIdx = static_cast<uint32>(w - lits_);
 			// other watched literal?
-			if (w->watched()) { other_ = nIdx; continue; }
+			if (w->flagged()) { other_ = nIdx; continue; }
 			// replace watch
-			lits_[idx].clearWatch();
-			w->watch();
+			lits_[idx].unflag();
+			w->flag();
 			// add new watch only w is not one of the atoms
 			// and keep previous watch if p is one of the atoms
 			if (nIdx != xPos_) { s.addWatch(~*w, this, (nIdx << 1) + (dir==1)); }
@@ -966,17 +973,15 @@ Constraint::PropResult LoopFormula::propagate(Solver& s, Literal p, uint32& data
 	}
 }
 void LoopFormula::reason(Solver& s, Literal p, LitVec& lits) {
-	uint32 os = lits.size();
 	// p = body: all literals in active clause
 	// p = atom: only bodies
 	for (Literal* it = begin() + (other_ == xPos_); !isSentinel(*it); ++it) {
 		if (*it != p) { lits.push_back(~*it); }
 	}
-	act_.setLbd(s.updateLearnt(p, &lits[0]+os, &lits[0]+lits.size(), act_.lbd()));
-	act_.bumpAct();
+	s.updateOnReason(act_, p, lits);
 }
 bool LoopFormula::minimize(Solver& s, Literal p, CCMinRecursive* rec) {
-	act_.bumpAct();
+	s.updateOnMinimize(act_);
 	for (Literal* it = begin() + (other_ == xPos_); !isSentinel(*it); ++it) {
 		if (*it != p && !s.ccMinimize(~*it, rec)) { return false; }
 	}
@@ -996,7 +1001,7 @@ bool LoopFormula::locked(const Solver& s) const {
 	return false;
 }
 uint32 LoopFormula::isOpen(const Solver& s, const TypeSet& xs, LitVec& freeLits) {
-	if (!xs.inSet(Constraint_t::learnt_loop) || otherIsSat(s)) {
+	if (!xs.inSet(Constraint_t::Loop) || otherIsSat(s)) {
 		return 0;
 	}
 	for (Literal* it = begin() + xPos_; !isSentinel(*it); ++it) {
@@ -1006,7 +1011,7 @@ uint32 LoopFormula::isOpen(const Solver& s, const TypeSet& xs, LitVec& freeLits)
 	for (Literal* it = xBegin(), *end = xEnd(); it != end; ++it) {
 		if (s.value(it->var()) == value_free) { freeLits.push_back(*it); }
 	}
-	return Constraint_t::learnt_loop;
+	return Constraint_t::Loop;
 }
 bool LoopFormula::simplify(Solver& s, bool) {
 	if (otherIsSat(s) || (other_ != xPos_ && (other_ = xPos_) != 0 && otherIsSat(s))) {
@@ -1020,15 +1025,15 @@ bool LoopFormula::simplify(Solver& s, bool) {
 		if (*it == lits_[xPos_]){ xPos_ = 0; }
 		for (GenericWatch* w; !isSentinel(*it); ++it) {
 			if (s.value(it->var()) == value_free) {
-				if (it->watched() && (w = s.getWatch(~*it, this)) != 0) {
+				if (it->flagged() && (w = s.getWatch(~*it, this)) != 0) {
 					w->data = (static_cast<uint32>(j - lits_) << 1) + (w->data&1);
 				}
 				*j++ = *it;
 			}
 			else if (s.isTrue(*it)) { detach(s); return true; }
-			else                    { assert(!it->watched() && "Constraint not propagated!"); }
+			else                    { assert(!it->flagged() && "Constraint not propagated!"); }
 		}
-		*j   = posLit(0);
+		*j   = lit_true();
 		end_ = static_cast<uint32>(j - lits_);
 	}
 	// simplify extra part
@@ -1039,17 +1044,17 @@ bool LoopFormula::simplify(Solver& s, bool) {
 	bool isClause = static_cast<uint32>(j - xBegin()) == 1;
 	if (isClause) { --j; }
 	if (j != end) { // size changed?
-		if (!str_)   { (end-1)->asUint() = 3u; str_ = 1u; }
+		if (!str_)   { (end-1)->rep() = 3u; str_ = 1u; }
 		if (isClause){
 			assert(xPos_ && *j == lits_[xPos_]);
-			if (!lits_[xPos_].watched()) { s.removeWatch(~*j, this); }
+			if (!lits_[xPos_].flagged()) { s.removeWatch(~*j, this); }
 			xPos_ = 0;
 		}
 		size_ = static_cast<uint32>((end = j) - lits_);
 	}
 	assert(!isClause || xPos_ == 0);
 	other_ = xPos_ + 1;
-	ClauseRep act = ClauseRep::create(begin(), end_ - 1, Constraint_t::learnt_loop);
+	ClauseRep act = ClauseRep::create(begin(), end_ - 1, Constraint_t::Loop);
 	if (act.isImp() && s.allowImplicit(act)) {
 		detach(s);
 		ClauseCreator::Result res;
