@@ -1,18 +1,18 @@
-// 
+//
 // Copyright (c) 2006-2011, Benjamin Kaufmann
-// 
-// This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
-// 
+//
+// This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/
+//
 // Clasp is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 2 of the License, or
 // (at your option) any later version.
-// 
+//
 // Clasp is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU General Public License
 // along with Clasp; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
@@ -36,8 +36,7 @@ WeightLitsRep WeightLitsRep::create(Solver& s, WeightLitVec& lits, weight_t boun
 	LitVec::size_type j = 0, other;
 	const weight_t MAX_W= std::numeric_limits<weight_t>::max();
 	for (LitVec::size_type i = 0; i != lits.size(); ++i) {
-		lits[i].first.clearWatch();
-		Literal x = lits[i].first;
+		Literal x = lits[i].first.unflag();
 		if (lits[i].second != 0 && s.topValue(x.var()) == value_free) {
 			if (lits[i].second < 0) {
 				lits[i].second = -lits[i].second;
@@ -57,8 +56,8 @@ WeightLitsRep WeightLitsRep::create(Solver& s, WeightLitVec& lits, weight_t boun
 			else {                  // complementary literals ~x x
 				for (other = 0; other != j && lits[other].first != ~x; ++other) { ; }
 				bound              -= lits[i].second; // decrease by min(w(~x), w(x)) ; assume w(~x) > w(x)
-				lits[other].second -= lits[i].second; // keep ~x, 
-				if (lits[other].second < 0) {         // actually, w(x) > w(~x), 
+				lits[other].second -= lits[i].second; // keep ~x,
+				if (lits[other].second < 0) {         // actually, w(x) > w(~x),
 					lits[other].first  = x;             // replace ~x with x
 					lits[other].second = -lits[other].second;
 					s.clearSeen(x.var());
@@ -108,8 +107,8 @@ WeightLitsRep WeightLitsRep::create(Solver& s, WeightLitVec& lits, weight_t boun
 bool WeightLitsRep::propagate(Solver& s, Literal W) {
 	if      (sat())  { return s.force(W); } // trivially SAT
 	else if (unsat()){ return s.force(~W);} // trivially UNSAT
-	else if (s.topValue(W.var()) == value_free) { 
-		return true; 
+	else if (s.topValue(W.var()) == value_free) {
+		return true;
 	}
 	// backward propagate
 	bool bpTrue = s.isTrue(W);
@@ -131,11 +130,12 @@ bool WeightLitsRep::propagate(Solver& s, Literal W) {
 /////////////////////////////////////////////////////////////////////////////////////////
 // WeightConstraint::WL
 /////////////////////////////////////////////////////////////////////////////////////////
+typedef Clasp::Atomic_t<uint32>::type RCType;
 WeightConstraint::WL::WL(uint32 s, bool shared, bool hasW) : sz(s), rc(shared), w(hasW) { }
 uint8* WeightConstraint::WL::address() { return reinterpret_cast<unsigned char*>(this) - (sizeof(uint32) * rc); }
 WeightConstraint::WL* WeightConstraint::WL::clone(){
 	if (shareable()) {
-		++*reinterpret_cast<Clasp::atomic<uint32>*>(address());
+		++*reinterpret_cast<RCType*>(address());
 		return this;
 	}
 	else {
@@ -147,47 +147,41 @@ WeightConstraint::WL* WeightConstraint::WL::clone(){
 }
 void WeightConstraint::WL::release() {
 	unsigned char* x = address();
-	if (!shareable() || --*reinterpret_cast<Clasp::atomic<uint32>*>(x) == 0) {
+	if (!shareable() || --*reinterpret_cast<RCType*>(x) == 0) {
 		::operator delete(x);
 	}
 }
 uint32 WeightConstraint::WL::refCount() const {
 	assert(shareable());
-	return *reinterpret_cast<const Clasp::atomic<uint32>*>(const_cast<WL*>(this)->address());
+	return *reinterpret_cast<const RCType*>(const_cast<WL*>(this)->address());
 }
 /////////////////////////////////////////////////////////////////////////////////////////
 // WeightConstraint
 /////////////////////////////////////////////////////////////////////////////////////////
 WeightConstraint::CPair WeightConstraint::create(Solver& s, Literal W, WeightLitVec& lits, weight_t bound, uint32 flags) {
-	if ((flags & create_eq_bound) == 0) {
-		return WeightConstraint::create(s, W, WeightLitsRep::create(s, lits, bound), flags);
-	}
-	WeightLitsRep rep = WeightLitsRep::create(s, lits, bound + 1);
+	bool const    eq  = (flags & create_eq_bound) != 0;
+	WeightLitsRep rep = WeightLitsRep::create(s, lits, bound + static_cast<int>(eq));
 	CPair res;
-	res.con[1] = WeightConstraint::createImpl(s, ~W, rep, flags);
-	if (res.ok()) {
+	if (eq) {
+		res.con[1] = WeightConstraint::doCreate(s, ~W, rep, flags);
 		rep.bound -= 1;
+		if (!res.ok()) { return res; }
 		// redo coefficient reduction
 		for (unsigned i = 0; i != rep.size && rep.lits[i].second > rep.bound; ++i) {
 			rep.reach -= rep.lits[i].second;
 			rep.reach += (rep.lits[i].second = rep.bound);
 		}
-		res.con[0] = WeightConstraint::createImpl(s, W, rep, flags);
 	}
-	return res;	
+	res.con[0] = WeightConstraint::doCreate(s, W, rep, flags);
+	return res;
 }
-WeightConstraint::CPair WeightConstraint::create(Solver& s, Literal W, WeightLitsRep rep, uint32 flags) {
+WeightConstraint::CPair WeightConstraint::create(Solver& s, Literal W, WeightLitsRep& rep, uint32 flags) {
 	CPair res;
-	res.con[0] = createImpl(s, W, rep, flags);
-	if ((flags & create_eq_bound) != 0 && res.ok()) {
-		// Adds constraints for W == (lits == bound)
-		++rep.bound;
-		res.con[1] = createImpl(s, ~W, rep, flags);
-	}
+	res.con[0] = doCreate(s, W, rep, flags);
 	return res;
 }
 
-WeightConstraint* WeightConstraint::createImpl(Solver& s, Literal W, WeightLitsRep& rep, uint32 flags) {
+WeightConstraint* WeightConstraint::doCreate(Solver& s, Literal W, WeightLitsRep& rep, uint32 flags) {
 	WeightConstraint* conflict = (WeightConstraint*)0x1;
 	bool addSat = (flags&create_sat) != 0 && rep.size;
 	if (!rep.propagate(s, W))                 { return conflict; }
@@ -198,7 +192,7 @@ WeightConstraint* WeightConstraint::createImpl(Solver& s, Literal W, WeightLitsR
 		bool disj = rep.bound == 1; // con == disjunction or con == conjunction
 		bool sat  = false;
 		clause.push_back(W ^ disj);
-		for (LitVec::size_type i = 0; i != rep.size; ++i) { 
+		for (LitVec::size_type i = 0; i != rep.size; ++i) {
 			bin[0] = ~clause[0];
 			bin[1] = rep.lits[i].first ^ disj;
 			if (bin[0] != ~bin[1]) {
@@ -224,16 +218,16 @@ WeightConstraint* WeightConstraint::createImpl(Solver& s, Literal W, WeightLitsR
 	void*  m    = 0;
 	WL*    sL   = 0;
 	if ((flags & create_no_share) != 0) {
-		nb        = ((nb + 3) / 4)*4;
-		m         = ::operator new (nb + wls);
-		sL        = new (reinterpret_cast<unsigned char*>(m)+nb) WL(size, false, hasW);
+		nb = ((nb + 3) / 4)*4;
+		m  = ::operator new (nb + wls);
+		sL = new (reinterpret_cast<unsigned char*>(m)+nb) WL(size, false, hasW);
 	}
 	else {
-		static_assert(sizeof(Clasp::atomic<uint32>) == sizeof(uint32), "Invalid size!");
-		m         = ::operator new(nb);
-		uint8* t  = (uint8*)::operator new(wls + sizeof(uint32));
-		*(new (t) Clasp::atomic<uint32>()) = 1;
-		sL        = new (t+sizeof(uint32)) WL(size, true, hasW);
+		static_assert(sizeof(RCType) == sizeof(uint32), "Invalid size!");
+		m = ::operator new(nb);
+		uint8* t = (uint8*)::operator new(wls + sizeof(uint32));
+		*(new (t) RCType) = 1;
+		sL = new (t+sizeof(uint32)) WL(size, true, hasW);
 	}
 	assert(m && (reinterpret_cast<uintp>(m) & 7u) == 0);
 	SharedContext*  ctx = (flags & create_no_freeze) == 0 ? const_cast<SharedContext*>(s.sharedContext()) : 0;
@@ -246,14 +240,14 @@ WeightConstraint* WeightConstraint::createImpl(Solver& s, Literal W, WeightLitsR
 	return c;
 }
 WeightConstraint::WeightConstraint(Solver& s, SharedContext* ctx, Literal W, const WeightLitsRep& rep, WL* out, uint32 act) {
+	typedef unsigned char Byte_t;
 	const bool hasW = rep.hasWeights();
 	lits_           = out;
 	active_         = act;
 	ownsLit_        = !out->shareable();
 	Literal* p      = lits_->lits;
-	Literal* h      = (Literal*)undo_;
+	Literal* h      = new (reinterpret_cast<Byte_t*>(undo_)) Literal(W);
 	weight_t w      = 1;
-	Var      mv     = W.var();
 	bound_[FFB_BTB]	= (rep.reach-rep.bound)+1; // ffb-btb
 	bound_[FTB_BFB]	= rep.bound;               // ftb-bfb
 	*p++            = ~W;                      // store constraint literal
@@ -263,20 +257,21 @@ WeightConstraint::WeightConstraint(Solver& s, SharedContext* ctx, Literal W, con
 		active_ = FFB_BTB+s.isFalse(W);
 	}
 	watched_        = 3u - (active_ != 3u || ctx == 0);
-	for (uint32 i = 0, j = 1, end = rep.size; i != end; ++i, ++j) {
-		*p++ = h[j] = rep.lits[i].first;         // store constraint literal
-		w    = rep.lits[i].second;               // followed by weight 
+	WeightLiteral*x = rep.lits;
+	for (uint32 sz = rep.size, j = 1; sz--; ++j, ++x) {
+		h    = new (h + 1) Literal(x->first);
+		*p++ = x->first;                         // store constraint literal
+		w    = x->second;                        // followed by weight
 		if (hasW) *p++= Literal::fromRep(w);     // if necessary
 		addWatch(s, j, FTB_BFB);                 // watches  lits[idx]
 		addWatch(s, j, FFB_BTB);                 // watches ~lits[idx]
-		if (ctx) ctx->setFrozen(h[j].var(), true);// exempt from variable elimination
-		mv   = std::max(mv, h[j].var());
+		if (ctx) ctx->setFrozen(h->var(), true); // exempt from variable elimination
 	}
-	if (hasW) s.requestData(mv+1);         // weight constraints can become unit more than once
 	// init heuristic
+	h -= rep.size;
 	uint32 off = active_ != NOT_ACTIVE;
-	h[0]       = W;
-	s.heuristic()->newConstraint(s, h+off, rep.size+(1-off), Constraint_t::static_constraint);
+	assert((void*)h == (void*)undo_);
+	s.heuristic()->newConstraint(s, h+off, rep.size+(1-off), Constraint_t::Static);
 	// init undo stack
 	up_                 = undoStart();     // undo stack is initially empty
 	undo_[0].data       = 0;
@@ -293,35 +288,37 @@ WeightConstraint::WeightConstraint(Solver& s, SharedContext* ctx, Literal W, con
 }
 
 WeightConstraint::WeightConstraint(Solver& s, const WeightConstraint& other) {
+	typedef unsigned char Byte_t;
 	lits_        = other.lits_->clone();
 	ownsLit_     = 0;
-	Literal* heu = (Literal*)undo_;
-	heu[0]       = ~lits_->lit(0);
+	Literal* heu = new (reinterpret_cast<Byte_t*>(undo_))Literal(~lits_->lit(0));
 	bound_[0]	   = other.bound_[0];
 	bound_[1]	   = other.bound_[1];
 	active_      = other.active_;
 	watched_     = other.watched_;
-	if (active_ == NOT_ACTIVE && s.value(heu[0].var()) == value_free) {
+	if (active_ == NOT_ACTIVE && s.value(heu->var()) == value_free) {
 		addWatch(s, 0, FTB_BFB);  // watch con in both phases
 		addWatch(s, 0, FFB_BTB);  // in order to allow for backpropagation
 	}
 	for (uint32 i = 1, end = size(); i < end; ++i) {
-		heu[i]      = lits_->lit(i);
-		if (s.value(heu[i].var()) == value_free) {
+		heu = new (heu + 1) Literal(lits_->lit(i));
+		if (s.value(heu->var()) == value_free) {
 			addWatch(s, i, FTB_BFB);  // watches  lits[i]
 			addWatch(s, i, FFB_BTB);  // watches ~lits[i]
 		}
 	}
 	// Initialize heuristic with literals (no weights) in constraint.
 	uint32 off = active_ != NOT_ACTIVE;
-	s.heuristic()->newConstraint(s, heu+off, size()-off, Constraint_t::static_constraint);
+	heu -= (size() - 1);
+	assert((void*)heu == (void*)undo_);
+	s.heuristic()->newConstraint(s, heu+off, size()-off, Constraint_t::Static);
 	// Init undo stack
 	std::memcpy(undo_, other.undo_, sizeof(UndoInfo)*(size()+isWeight()));
 	up_ = other.up_;
 }
 
 WeightConstraint::~WeightConstraint() {}
-Constraint* WeightConstraint::cloneAttach(Solver& other) { 
+Constraint* WeightConstraint::cloneAttach(Solver& other) {
 	void* m = ::operator new(sizeof(WeightConstraint) + (size()+isWeight())*sizeof(UndoInfo));
 	return new (m) WeightConstraint(other, *this);
 }
@@ -333,15 +330,15 @@ bool WeightConstraint::integrateRoot(Solver& s) {
 	uint32 np  = 0;
 	for (uint32 i = 0, end = size(); i != end; ++i) {
 		Var v = lits_->var(i);
-		if (s.value(v) != value_free && (vDL = s.level(v)) != 0) { 
-			++np; 
+		if (s.value(v) != value_free && (vDL = s.level(v)) != 0) {
+			++np;
 			s.markSeen(v);
 			low = std::min(low, vDL);
 		}
 	}
 	// propagate assigned literals in assignment order
 	const LitVec& trail = s.trail();
-	const uint32  end   = trail.size() - s.queueSize();
+	const uint32  end   = sizeVec(trail) - s.queueSize();
 	GenericWatch* w     = 0;
 	for (uint32 i = s.levelStart(low); i != end && np; ++i) {
 		Literal p = trail[i];
@@ -371,28 +368,28 @@ void WeightConstraint::destroy(Solver* s, bool detach) {
 			s->removeWatch( lits_->lit(i), this );
 			s->removeWatch(~lits_->lit(i), this );
 		}
-		for (uint32 i = s->decisionLevel(); i; --i) {
-			s->removeUndoWatch(i, this);
+		for (uint32 last = 0, dl; (dl = highestUndoLevel(*s)) != 0; --up_) {
+			if (dl != last) { s->removeUndoWatch(last = dl, this); }
 		}
 	}
 	if (ownsLit_ == 0) { lits_->release(); }
-	void* mem    = static_cast<Constraint*>(this);
+	void* mem = static_cast<Constraint*>(this);
 	this->~WeightConstraint();
-	::operator delete(mem);	
+	::operator delete(mem);
 }
 
-void WeightConstraint::setBpIndex(uint32 n){ 
-	if (isWeight()) undo_[0].data = (n<<1)+(undo_[0].data&1); 
+void WeightConstraint::setBpIndex(uint32 n){
+	if (isWeight()) undo_[0].data = (n<<1)+(undo_[0].data&1);
 }
 
 // Returns the numerical highest decision level watched by this constraint.
 uint32 WeightConstraint::highestUndoLevel(Solver& s) const {
-	return up_ != undoStart() 
+	return up_ != undoStart()
 		? s.level(lits_->var(undoTop().idx()))
 		: 0;
 }
 
-// Updates the bound of sub-constraint c and adds the literal at index idx to the 
+// Updates the bound of sub-constraint c and adds the literal at index idx to the
 // undo stack. If the current decision level is not watched, an undo watch is added
 // so that the bound can be adjusted once the solver backtracks.
 void WeightConstraint::updateConstraint(Solver& s, uint32 idx, ActiveConstraint c) {
@@ -408,11 +405,11 @@ void WeightConstraint::updateConstraint(Solver& s, uint32 idx, ActiveConstraint
 
 // Since clasp uses an eager assignment strategy where literals are assigned as soon
 // as they are added to the propagation queue, we distinguish processed from unprocessed literals.
-// Processed literals are those for which propagate was already called and the corresponding bound 
-// was updated; they are flagged in updateConstraint(). 
+// Processed literals are those for which propagate was already called and the corresponding bound
+// was updated; they are flagged in updateConstraint().
 // Unprocessed literals are either free or were not yet propagated. During propagation
 // we treat all unprocessed literals as free. This way, conflicts are detected early.
-// Consider: x :- 3 [a=3, b=2, c=1,d=1] and PropQ: b, ~Body, c. 
+// Consider: x :- 3 [a=3, b=2, c=1,d=1] and PropQ: b, ~Body, c.
 // Initially b, ~Body, c are unprocessed and the bound is 3.
 // Step 1: propagate(b)    : b is marked as processed and bound is reduced to 1.
 //   Now, although we already know that the body is false, we do not backpropagate yet
@@ -421,8 +418,8 @@ void WeightConstraint::updateConstraint(Solver& s, uint32 idx, ActiveConstraint
 // Step 2: propagate(~Body): ~body is marked as processed and bound is reduced to 0.
 //   Since the body is now part of our reason set, we can start backpropagation.
 //   First we assign the unprocessed and free literal ~a. Literal ~b is skipped, because
-//   its complementary literal was already successfully processed. Finally, we force 
-//   the unprocessed but false literal ~c to true. This will generate a conflict and 
+//   its complementary literal was already successfully processed. Finally, we force
+//   the unprocessed but false literal ~c to true. This will generate a conflict and
 //   propagation is stopped. Without the distinction between processed and unprocessed
 //   lits we would have to skip ~c. We would then have to manually trigger the conflict
 //   {b, ~Body, c} in step 3, when propagate(c) sets the bound to -1.
@@ -432,9 +429,9 @@ Constraint::PropResult WeightConstraint::propagate(Solver& s, Literal, uint32& d
 	const uint32   idx  = d >> 1;
 	Literal body        = lit(0, c);
 	if ( uint32(c^1) == active_ || s.isTrue(body) ) {
-		// the other constraint is active or this constraint is already satisfied; 
+		// the other constraint is active or this constraint is already satisfied;
 		// nothing to do
-		return PropResult(true, true);		
+		return PropResult(true, true);
 	}
 	if (idx == 0 && s.decisionLevel() == s.rootLevel() && watched_ == 3u) {
 		watched_ = c;
@@ -472,7 +469,7 @@ Constraint::PropResult WeightConstraint::propagate(Solver& s, Literal, uint32& d
 }
 
 // Builds the reason for p from the undo stack of this constraint.
-// The reason will only contain literals that were processed by the 
+// The reason will only contain literals that were processed by the
 // active sub-constraint.
 void WeightConstraint::reason(Solver& s, Literal p, LitVec& r) {
 	assert(active_ != NOT_ACTIVE);
@@ -507,7 +504,7 @@ bool WeightConstraint::minimize(Solver& s, Literal p, CCMinRecursive* rec) {
 	return true;
 }
 
-// undoes processed assignments 
+// undoes processed assignments
 void WeightConstraint::undoLevel(Solver& s) {
 	setBpIndex(1);
 	for (UndoInfo u; up_ != undoStart() && s.value(lits_->var((u=undoTop()).idx())) == value_free;) {
@@ -539,7 +536,7 @@ bool WeightConstraint::simplify(Solver& s, bool) {
 	else if (s.value(lits_->var(0)) != value_free && (active_ == NOT_ACTIVE || isWeight())) {
 		if (active_ == NOT_ACTIVE) {
 			Literal W = ~lits_->lit(0);
-			active_   = FFB_BTB+s.isFalse(W);	
+			active_   = FFB_BTB+s.isFalse(W);
 		}
 		for (uint32 i = 0, end = size(); i != end; ++i) {
 			s.removeWatch(lit(i, (ActiveConstraint)active_), this);
