@@ -1,18 +1,18 @@
-// 
+//
 // Copyright (c) 2006, Benjamin Kaufmann
-// 
-// This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
-// 
+//
+// This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/
+//
 // Clasp is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 2 of the License, or
 // (at your option) any later version.
-// 
+//
 // Clasp is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU General Public License
 // along with Clasp; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
@@ -26,7 +26,7 @@ using namespace Clasp::mt;
 class ClauseCreatorTest : public CppUnit::TestFixture {
 
   CPPUNIT_TEST_SUITE(ClauseCreatorTest);
-	CPPUNIT_TEST(testEmptyClauseIsFalse);	
+	CPPUNIT_TEST(testEmptyClauseIsFalse);
 	CPPUNIT_TEST(testFactsAreAsserted);
 	CPPUNIT_TEST(testTopLevelSATClausesAreNotAdded);
 	CPPUNIT_TEST(testTopLevelFalseLitsAreRemoved);
@@ -47,7 +47,7 @@ class ClauseCreatorTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testCreateLearntClauseConflict);
 	CPPUNIT_TEST(testCreateNonAssertingLearntClauseAsserting);
 	CPPUNIT_TEST(testCreateBogusUnit);
-	
+
 	CPPUNIT_TEST(testInitWatches);
 	CPPUNIT_TEST(testIntegrateEmpty);
 	CPPUNIT_TEST(testIntegrateUnit);
@@ -67,24 +67,24 @@ class ClauseCreatorTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testIntegrateUnsat);
 	CPPUNIT_TEST(testIntegrateAssertingBelowBT);
 	CPPUNIT_TEST(testIntegrateConflictBelowBT);
-	
+
 	CPPUNIT_TEST(testFactsAreRemovedFromLearnt);
-	CPPUNIT_TEST_SUITE_END();	
+	CPPUNIT_TEST_SUITE_END();
 public:
 	ClauseCreatorTest() {
-		a = posLit(ctx.addVar(Var_t::atom_var));
-		b = posLit(ctx.addVar(Var_t::atom_var));
-		c = posLit(ctx.addVar(Var_t::atom_var));
-		d = posLit(ctx.addVar(Var_t::atom_var));
-		e = posLit(ctx.addVar(Var_t::atom_var));
-		f = posLit(ctx.addVar(Var_t::atom_var));
+		a = posLit(ctx.addVar(Var_t::Atom));
+		b = posLit(ctx.addVar(Var_t::Atom));
+		c = posLit(ctx.addVar(Var_t::Atom));
+		d = posLit(ctx.addVar(Var_t::Atom));
+		e = posLit(ctx.addVar(Var_t::Atom));
+		f = posLit(ctx.addVar(Var_t::Atom));
 		creator.setSolver(*ctx.master());
 		ctx.startAddConstraints(1);
 	}
 	void testEmptyClauseIsFalse() {
 		CPPUNIT_ASSERT_EQUAL(false, (bool)creator.start().end());
 	}
-	
+
 	void testFactsAreAsserted() {
 		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.start().add(a).end());
 		CPPUNIT_ASSERT_EQUAL(true, solver().isTrue(a));
@@ -111,7 +111,7 @@ public:
 		CPPUNIT_ASSERT_EQUAL(1u, ctx.numTernary());
 	}
 	void testAddGenericClause() {
-		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.start().add(a).add(b).add(c).add(d).end());		
+		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.start().add(a).add(b).add(c).add(d).end());
 		CPPUNIT_ASSERT_EQUAL(1u, ctx.numConstraints());
 	}
 
@@ -121,7 +121,7 @@ public:
 		solver().assume(~c);
 		solver().assume(~d);
 		solver().propagate();
-		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.start(Constraint_t::learnt_conflict).add(a)
+		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.start(Constraint_t::Conflict).add(a)
 			.add(b).add(c).add(d).end());
 
 		CPPUNIT_ASSERT_EQUAL(true, solver().isTrue(a));
@@ -134,8 +134,8 @@ public:
 		solver().assume(~b);
 		solver().assume(~c);
 		solver().assume(~d);
-		
-		creator.start(Constraint_t::learnt_conflict).add(a).add(b).add(d).add(c);
+
+		creator.start(Constraint_t::Conflict).add(a).add(b).add(d).add(c);
 		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.end());	// asserts a
 		solver().undoUntil(2);	// clear a and d
 		solver().assume(~d);		// hopefully d was watched.
@@ -150,32 +150,32 @@ public:
 			}
 			std::vector<LitVec::size_type> clSizes_;
 			std::vector<ConstraintType> clTypes_;
-		}*heu = new FakeHeu;
-		solver().setHeuristic(heu);
+		}heu;
+		solver().setHeuristic(&heu, Ownership_t::Retain);
 		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.start().add(a).add(b).add(c).add(d).end());
 		ctx.endInit();
 		solver().assume(a);
 		solver().assume(b);
 		solver().propagate();
 
-		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.start(Constraint_t::learnt_conflict).add(c)
+		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.start(Constraint_t::Conflict).add(c)
 			.add(~a).add(~b).end());
 
-		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.start(Constraint_t::learnt_loop).add(c)
+		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.start(Constraint_t::Loop).add(c)
 			.add(~a).add(~b).end(0));
 
-		CPPUNIT_ASSERT_EQUAL(uint32(3), (uint32)heu->clSizes_.size());
-		CPPUNIT_ASSERT_EQUAL(uint32(4), (uint32)heu->clSizes_[0]);
-		CPPUNIT_ASSERT_EQUAL(uint32(3), (uint32)heu->clSizes_[1]);
-		CPPUNIT_ASSERT_EQUAL(uint32(3), (uint32)heu->clSizes_[2]);
+		CPPUNIT_ASSERT_EQUAL(uint32(3), (uint32)heu.clSizes_.size());
+		CPPUNIT_ASSERT_EQUAL(uint32(4), (uint32)heu.clSizes_[0]);
+		CPPUNIT_ASSERT_EQUAL(uint32(3), (uint32)heu.clSizes_[1]);
+		CPPUNIT_ASSERT_EQUAL(uint32(3), (uint32)heu.clSizes_[2]);
 
-		CPPUNIT_ASSERT_EQUAL(Constraint_t::static_constraint, heu->clTypes_[0]);
-		CPPUNIT_ASSERT_EQUAL(Constraint_t::learnt_conflict, heu->clTypes_[1]);
-		CPPUNIT_ASSERT_EQUAL(Constraint_t::learnt_loop, heu->clTypes_[2]);
+		CPPUNIT_ASSERT_EQUAL(Constraint_t::Static, heu.clTypes_[0]);
+		CPPUNIT_ASSERT_EQUAL(Constraint_t::Conflict, heu.clTypes_[1]);
+		CPPUNIT_ASSERT_EQUAL(Constraint_t::Loop, heu.clTypes_[2]);
 	}
 
 	void testCreatorAddLitBug() {
-		creator.start(Constraint_t::learnt_conflict).add(a);
+		creator.start(Constraint_t::Conflict).add(a);
 		solver().assume(b) && solver().propagate();
 		creator.add(~b);
 		solver().assume(c) && solver().propagate();
@@ -202,7 +202,7 @@ public:
 		ctx.endInit();
 		solver().assume(a); solver().propagate();
 		solver().assume(b); solver().propagate();
-		creator.start(Constraint_t::learnt_loop);
+		creator.start(Constraint_t::Loop);
 		creator.add(~a).add(~b).add(~b).add(~a).add(c).add(d).prepare(true);
 		CPPUNIT_ASSERT(creator.size() == 4);
 		CPPUNIT_ASSERT_EQUAL(c, creator[0]);
@@ -211,7 +211,7 @@ public:
 
 	void testCreatorSimplifyBug() {
 		LitVec clause;
-		clause.push_back(negLit(0));
+		clause.push_back(lit_false());
 		clause.push_back(a);
 		clause.push_back(b);
 		ClauseCreator::prepare(*ctx.master(), clause, ClauseCreator::clause_force_simplify);
@@ -222,7 +222,7 @@ public:
 		ctx.endInit();
 		solver().assume(a); solver().propagate();
 		solver().assume(b); solver().propagate();
-		creator.start(Constraint_t::learnt_loop);
+		creator.start(Constraint_t::Loop);
 		creator.add(~a).add(~b).add(c).add(d);
 		CPPUNIT_ASSERT(creator.end());
 		CPPUNIT_ASSERT_EQUAL(c, creator[0]);
@@ -232,7 +232,7 @@ public:
 		solver().undoUntil(0);
 		// test with a short clause
 		solver().assume(a); solver().propagate();
-		creator.start(Constraint_t::learnt_loop);
+		creator.start(Constraint_t::Loop);
 		creator.add(~a).add(b).add(c);
 		CPPUNIT_ASSERT(creator.end());
 		CPPUNIT_ASSERT_EQUAL(b, creator[0]);
@@ -246,7 +246,7 @@ public:
 		solver().assume(d); solver().propagate();                     // level 3
 		solver().assume(f); solver().propagate();                     // level 4
 
-		creator.start(Constraint_t::learnt_loop);
+		creator.start(Constraint_t::Loop);
 		creator.add(~c).add(~a).add(~d).add(~b); // 2 1 3 1
 		CPPUNIT_ASSERT_EQUAL(false, (bool)creator.end());
 		// make sure we watch highest levels, i.e. 3 and 2
@@ -254,12 +254,12 @@ public:
 		CPPUNIT_ASSERT_EQUAL(~c, creator[1]);
 		CPPUNIT_ASSERT(solver().numLearntConstraints() == 0);
 
-		
+
 		solver().undoUntil(0);
 		// test with a short clause
 		solver().assume(a); solver().propagate();// level 1
 		solver().assume(c); solver().propagate();// level 2
-		creator.start(Constraint_t::learnt_loop);
+		creator.start(Constraint_t::Loop);
 		creator.add(~a).add(~c);
 		CPPUNIT_ASSERT_EQUAL(false, (bool)creator.end());
 		CPPUNIT_ASSERT_EQUAL(~c, creator[0]);
@@ -271,7 +271,7 @@ public:
 		ctx.endInit();
 		solver().assume(a); solver().force(b,0); solver().propagate();// level 1
 		solver().assume(c); solver().propagate();                     // level 2
-		creator.start(Constraint_t::learnt_loop);
+		creator.start(Constraint_t::Loop);
 		creator.add(~c).add(~a).add(d).add(~b);                       // 2 1 Free 1
 		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.end());
 		// make sure we watch the right lits, i.e. d (free) and ~c (highest DL)
@@ -284,7 +284,7 @@ public:
 		solver().reduceLearnts(1.0f);
 		solver().assume(a); solver().force(b,0); solver().propagate();// level 1
 		solver().assume(c); solver().propagate();                     // level 2
-		creator.start(Constraint_t::learnt_loop);
+		creator.start(Constraint_t::Loop);
 		creator.add(~c).add(~a).add(d);                               // 2 1 Free
 		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.end());
 		// make sure we watch the right lits, i.e. d (free) and ~c (highest DL)
@@ -299,8 +299,8 @@ public:
 		solver().force(~d,0) && solver().propagate();
 		solver().assume(~c) && solver().propagate();
 		CPPUNIT_ASSERT(solver().decisionLevel() == 3);
-		
-		creator.start(Constraint_t::learnt_other).add(d).add(b).add(c).add(a);
+
+		creator.start(Constraint_t::Other).add(d).add(b).add(c).add(a);
 		CPPUNIT_ASSERT(ClauseCreator::status(solver(), &creator.lits()[0], &creator.lits()[0] + creator.size()) == ClauseCreator::status_sat);
 
 		ClauseCreator::Result r = creator.end();
@@ -323,8 +323,8 @@ public:
 		ClauseCreator::prepare(solver(), temp, 0);
 		CPPUNIT_ASSERT(temp[0] == d);
 		CPPUNIT_ASSERT(temp[1] == a);
-		
-		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
+
+		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::Other));
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, ClauseCreator::clause_no_add);
 		temp.clear();
 		r.local->clause()->toLits(temp);
@@ -337,7 +337,7 @@ public:
 		LitVec cl;
 		solver().assume(~a) && solver().propagate();
 		solver().pushRootLevel(solver().decisionLevel());
-		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
+		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::Other));
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, 0);
 		CPPUNIT_ASSERT_EQUAL(false, r.ok());
 		CPPUNIT_ASSERT_EQUAL(true, solver().hasConflict());
@@ -351,11 +351,11 @@ public:
 		solver().pushRootLevel(solver().decisionLevel());
 		solver().assume(e) && solver().propagate();
 
-		LitVec cl; 
+		LitVec cl;
 		// ~a ~b ~c f -> Unit: f@3
 		cl.push_back(~a); cl.push_back(f);
 		cl.push_back(~c); cl.push_back(~b);
-		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
+		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::Other));
 		ClauseCreator::integrate(solver(), p, 0);
 		CPPUNIT_ASSERT(solver().isTrue(f));
 		CPPUNIT_ASSERT(solver().decisionLevel() == solver().rootLevel());
@@ -371,9 +371,9 @@ public:
 
 	void testIntegrateUnitSAT() {
 		solver().assume(a) && solver().propagate();
-		LitVec cl; 
+		LitVec cl;
 		cl.push_back(a);
-		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
+		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::Other));
 		ClauseCreator::integrate(solver(), p, 0);
 		CPPUNIT_ASSERT(solver().isTrue(a));
 		CPPUNIT_ASSERT(solver().decisionLevel() == 0);
@@ -384,11 +384,11 @@ public:
 		solver().assume(b) && solver().propagate();
 		solver().assume(c) && solver().propagate();
 		solver().force(d, 0) && solver().propagate();
-		
+
 		// ~a ~b ~c ~d -> conflicting@3
 		LitVec cl;
 		cl.push_back(~a); cl.push_back(~c); cl.push_back(~b); cl.push_back(~d);
-		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
+		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::Other));
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, 0);
 		CPPUNIT_ASSERT(!r.ok());
 		CPPUNIT_ASSERT(r.local != 0);
@@ -400,11 +400,11 @@ public:
 		solver().assume(b) && solver().propagate();
 		solver().assume(c) && solver().propagate();
 		solver().assume(d) && solver().propagate();
-		
+
 		// ~a ~b ~c -> Conflict @3
 		LitVec cl;
 		cl.push_back(~a); cl.push_back(~c); cl.push_back(~b);
-		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
+		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::Other));
 		ClauseCreator::integrate(solver(), p, 0);
 		CPPUNIT_ASSERT(solver().decisionLevel() == uint32(2));
 	}
@@ -418,7 +418,7 @@ public:
 		// ~a ~b ~c -> Conflict @3, Asserting @2
 		LitVec cl;
 		cl.push_back(~a); cl.push_back(~c); cl.push_back(~b);
-		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
+		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::Other));
 		ClauseCreator::integrate(solver(), p, 0);
 		CPPUNIT_ASSERT(solver().decisionLevel() == uint32(3));
 		solver().popRootLevel();
@@ -433,19 +433,19 @@ public:
 		solver().assume(~c) && solver().propagate();
 		solver().assume(~d) && solver().propagate();
 		solver().pushRootLevel(solver().decisionLevel());
-		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
+		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::Other));
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, ClauseCreator::clause_explicit);
 		CPPUNIT_ASSERT_EQUAL(false, r.ok());
 		CPPUNIT_ASSERT_EQUAL(uint32(1), solver().numLearntConstraints());
 	}
-	
+
 	void testIntegrateSATBug1() {
 		LitVec cl;
 		cl.push_back(d); cl.push_back(b); cl.push_back(a); cl.push_back(c);
 		solver().assume(~a) && solver().propagate();
 		solver().assume(b) && solver().propagate();
 		solver().assume(~d) && solver().propagate();
-		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
+		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::Other));
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, 0);
 		CPPUNIT_ASSERT_EQUAL(true, r.ok());
 		CPPUNIT_ASSERT(3u == solver().numAssignedVars());
@@ -457,7 +457,7 @@ public:
 		solver().assume(b) && solver().propagate();
 		solver().assume(~d) && solver().propagate();
 		solver().force(c,0) && solver().propagate();
-		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
+		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::Other));
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, 0);
 		CPPUNIT_ASSERT_EQUAL(true, r.ok());
 	}
@@ -470,9 +470,9 @@ public:
 		solver().force(~d,0) && solver().propagate();
 		solver().assume(c) && solver().propagate();
 		CPPUNIT_ASSERT(solver().decisionLevel() == 3);
-		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
+		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::Other));
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, ClauseCreator::clause_not_sat);
-		
+
 		CPPUNIT_ASSERT_EQUAL(true, r.ok());
 		CPPUNIT_ASSERT(solver().decisionLevel() == 2);
 		CPPUNIT_ASSERT(solver().isTrue(c));
@@ -483,7 +483,7 @@ public:
 		cl.push_back(a); cl.push_back(b);
 		solver().force(~a, 0);
 		solver().assume(b);
-		SharedLiterals* p(SharedLiterals::newShareable(cl,Constraint_t::learnt_other));
+		SharedLiterals* p(SharedLiterals::newShareable(cl,Constraint_t::Other));
 		CPPUNIT_ASSERT((ClauseCreator::status(solver(), p->begin(), p->end()) & ClauseCreator::status_unit) != 0);
 		ClauseCreator::integrate(solver(), p, ClauseCreator::clause_explicit);
 	}
@@ -491,7 +491,7 @@ public:
 	void testIntegrateKnownOrderBug() {
 		LitVec cl;
 		cl.push_back(a);
-		SharedLiterals* p(SharedLiterals::newShareable(cl,Constraint_t::learnt_other));
+		SharedLiterals* p(SharedLiterals::newShareable(cl,Constraint_t::Other));
 		CPPUNIT_ASSERT((ClauseCreator::status(solver(), p->begin(), p->end()) & ClauseCreator::status_unit) != 0);
 		ClauseCreator::integrate(solver(), p, ClauseCreator::clause_no_prepare);
 		CPPUNIT_ASSERT(solver().isTrue(a));
@@ -500,7 +500,7 @@ public:
 	void testIntegrateNotConflictingBug() {
 		LitVec cl;
 		cl.push_back(a); cl.push_back(b);
-		SharedLiterals* p(SharedLiterals::newShareable(cl,Constraint_t::learnt_other));
+		SharedLiterals* p(SharedLiterals::newShareable(cl,Constraint_t::Other));
 		solver().assume(~a) && solver().propagate();
 		solver().force(~b,0)&& solver().propagate();
 		CPPUNIT_ASSERT((ClauseCreator::status(solver(), p->begin(), p->end()) & ClauseCreator::status_unsat) != 0);
@@ -512,7 +512,7 @@ public:
 		LitVec cl;
 		cl.push_back(a); cl.push_back(b); cl.push_back(c);
 		cl.push_back(d); cl.push_back(e); cl.push_back(f);
-		SharedLiterals* p(SharedLiterals::newShareable(cl,Constraint_t::learnt_other));
+		SharedLiterals* p(SharedLiterals::newShareable(cl,Constraint_t::Other));
 		solver().force(~d, 0) && solver().propagate();
 		solver().assume(~a)   && solver().propagate();
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, ClauseCreator::clause_no_add);
@@ -529,7 +529,7 @@ public:
 		solver().assume(b) && solver().propagate();
 		solver().assume(~d) && solver().propagate();
 		do {
-			SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
+			SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::Other));
 			ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, ClauseCreator::clause_not_sat);
 			CPPUNIT_ASSERT_EQUAL(true, r.ok());
 			CPPUNIT_ASSERT(solver().numAssignedVars() == 3);
@@ -542,7 +542,7 @@ public:
 		solver().assume(b);
 		LitVec cl;
 		cl.push_back(a);
-		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
+		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::Other));
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, 0);
 		CPPUNIT_ASSERT_EQUAL(false, r.ok());
 		CPPUNIT_ASSERT(0 == r.local);
@@ -558,7 +558,7 @@ public:
 		// ~a ~b ~c -> Conflict @3, Asserting @2
 		LitVec cl;
 		cl.push_back(~a); cl.push_back(~c); cl.push_back(~b);
-		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
+		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::Other));
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, 0);
 		CPPUNIT_ASSERT_EQUAL(false, r.ok());
 		solver().backtrack();
@@ -577,7 +577,7 @@ public:
 		// ~a ~b ~c -> Conflict @2
 		LitVec cl;
 		cl.push_back(~a); cl.push_back(~c); cl.push_back(~b);
-		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
+		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::Other));
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, 0);
 		CPPUNIT_ASSERT_EQUAL(false, r.ok());
 		CPPUNIT_ASSERT_EQUAL(true, solver().resolveConflict());
@@ -590,7 +590,7 @@ public:
 		ctx.addUnary(a);
 		ctx.endInit();
 		solver().assume(~b) && solver().propagate();
-		creator.start(Constraint_t::learnt_conflict);
+		creator.start(Constraint_t::Conflict);
 		creator.add(b).add(c).add(~a).end();
 
 		CPPUNIT_ASSERT(1u == ctx.numLearntShort());
@@ -604,4 +604,4 @@ private:
 	Literal a,b,c,d,e,f;
 };
 CPPUNIT_TEST_SUITE_REGISTRATION(ClauseCreatorTest);
-} } 
+} }
