@@ -1,18 +1,18 @@
-// 
+//
 // Copyright (c) 2006, Benjamin Kaufmann
-// 
-// This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
-// 
+//
+// This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/
+//
 // Clasp is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 2 of the License, or
 // (at your option) any later version.
-// 
+//
 // Clasp is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU General Public License
 // along with Clasp; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
@@ -25,18 +25,17 @@ namespace Clasp { namespace Test {
 class LiteralTest : public CppUnit::TestFixture {
   CPPUNIT_TEST_SUITE(LiteralTest);
 	CPPUNIT_TEST(testCtor);
-	CPPUNIT_TEST(testIndex);
-	CPPUNIT_TEST(testIndexIgnoresWatchedFlag);
-	CPPUNIT_TEST(testFromIndex);
-	CPPUNIT_TEST(testWatchedFlag);
-	CPPUNIT_TEST(testWatchedFlagCopy);
+	CPPUNIT_TEST(testId);
+	CPPUNIT_TEST(testIdIgnoresFlag);
+	CPPUNIT_TEST(testFromId);
+	CPPUNIT_TEST(testFlag);
+	CPPUNIT_TEST(testFlagCopy);
 	CPPUNIT_TEST(testComplement);
-	CPPUNIT_TEST(testComplementIsNotWatched);
+	CPPUNIT_TEST(testComplementIsNotFlagged);
 	CPPUNIT_TEST(testEquality);
 	CPPUNIT_TEST(testValue);
 	CPPUNIT_TEST(testLess);
 
-	CPPUNIT_TEST(testAntecedentAssumptions);
 	CPPUNIT_TEST(testAntecedentNullPointer);
 	CPPUNIT_TEST(testAntecedentPointer);
 	CPPUNIT_TEST(testAntecedentBin);
@@ -44,17 +43,17 @@ class LiteralTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST_SUITE_END();
 public:
 	LiteralTest() {
-		min = posLit(0), mid = posLit(varMax / 2),  max = posLit(varMax - 1);
+		min = lit_true(), mid = posLit(varMax / 2),  max = posLit(varMax - 1);
 	}
 	void testCtor() {
 		Literal p, q(42, true);
 		CPPUNIT_ASSERT_EQUAL(Var(0), p.var());
 		CPPUNIT_ASSERT_EQUAL(false, p.sign());
-		CPPUNIT_ASSERT_EQUAL(false, p.watched());
+		CPPUNIT_ASSERT_EQUAL(false, p.flagged());
 
 		CPPUNIT_ASSERT_EQUAL(Var(42), q.var());
 		CPPUNIT_ASSERT_EQUAL(true, q.sign());
-		CPPUNIT_ASSERT_EQUAL(false, q.watched());
+		CPPUNIT_ASSERT_EQUAL(false, q.flagged());
 
 		Literal x = posLit(7);
 		Literal y = negLit(7);
@@ -62,46 +61,43 @@ public:
 		CPPUNIT_ASSERT_EQUAL(false, x.sign());
 		CPPUNIT_ASSERT_EQUAL(true, y.sign());
 	}
-	void testIndex() {
-		uint32 minIdx	= min.index();
-		uint32 maxIdx	= max.index();
-		uint32 midIdx	= mid.index();
+	void testId() {
+		uint32 minIdx	= min.id();
+		uint32 maxIdx	= max.id();
+		uint32 midIdx	= mid.id();
 
 		CPPUNIT_ASSERT_EQUAL(uint32(0), minIdx);
-		CPPUNIT_ASSERT_EQUAL(uint32(1), (~min).index());
-		
+		CPPUNIT_ASSERT_EQUAL(uint32(1), (~min).id());
+
 		CPPUNIT_ASSERT_EQUAL(uint32((max.var()*2)), maxIdx);
-		CPPUNIT_ASSERT_EQUAL(uint32((max.var()*2)+1), (~max).index());
-		
+		CPPUNIT_ASSERT_EQUAL(uint32((max.var()*2)+1), (~max).id());
+
 		CPPUNIT_ASSERT_EQUAL(uint32((mid.var()*2)), midIdx);
-		CPPUNIT_ASSERT_EQUAL(uint32((mid.var()*2)+1), (~mid).index());
+		CPPUNIT_ASSERT_EQUAL(uint32((mid.var()*2)+1), (~mid).id());
 
-		CPPUNIT_ASSERT_EQUAL( minIdx, index(min.var()) );
-		CPPUNIT_ASSERT_EQUAL( maxIdx, index(max.var()) );
-		CPPUNIT_ASSERT_EQUAL( midIdx, index(mid.var()) );
 	}
-	void testIndexIgnoresWatchedFlag() {
+	void testIdIgnoresFlag() {
 		Literal maxW = max;
-		maxW.watch();
-		CPPUNIT_ASSERT_EQUAL(max.index(), maxW.index());
+		maxW.flag();
+		CPPUNIT_ASSERT_EQUAL(max.id(), maxW.id());
 	}
-	void testFromIndex() {
-		CPPUNIT_ASSERT(min == Literal::fromIndex(min.index()));
-		CPPUNIT_ASSERT(max == Literal::fromIndex(max.index()));
-		CPPUNIT_ASSERT(mid == Literal::fromIndex(mid.index()));
+	void testFromId() {
+		CPPUNIT_ASSERT(min == Literal::fromId(min.id()));
+		CPPUNIT_ASSERT(max == Literal::fromId(max.id()));
+		CPPUNIT_ASSERT(mid == Literal::fromId(mid.id()));
 	}
-	void testWatchedFlag() {
+	void testFlag() {
 		Literal p = posLit(42);
-		p.watch();
-		CPPUNIT_ASSERT_EQUAL(true, p.watched());
-		p.clearWatch();
-		CPPUNIT_ASSERT_EQUAL(false, p.watched());
+		p.flag();
+		CPPUNIT_ASSERT_EQUAL(true, p.flagged());
+		p.unflag();
+		CPPUNIT_ASSERT_EQUAL(false, p.flagged());
 	}
-	void testWatchedFlagCopy() {
+	void testFlagCopy() {
 		Literal p = posLit(42);
-		p.watch();
+		p.flag();
 		Literal q = p;
-		CPPUNIT_ASSERT_EQUAL(true, q.watched());
+		CPPUNIT_ASSERT_EQUAL(true, q.flagged());
 	}
 	void testComplement() {
 		Literal lit = posLit(7);
@@ -111,11 +107,11 @@ public:
 		CPPUNIT_ASSERT_EQUAL(true, cLit.sign());
 		CPPUNIT_ASSERT_EQUAL(true, lit == ~~lit);
 	}
-	void testComplementIsNotWatched() {
+	void testComplementIsNotFlagged() {
 		Literal lit = posLit(7);
-		lit.watch();
+		lit.flag();
 		Literal cLit = ~lit;
-		CPPUNIT_ASSERT_EQUAL(false, cLit.watched());
+		CPPUNIT_ASSERT_EQUAL(false, cLit.flagged());
 	}
 
 	void testEquality() {
@@ -128,10 +124,10 @@ public:
 	}
 
 	void testValue() {
-		CPPUNIT_ASSERT_EQUAL(value_true, trueValue(posLit(0)));
-		CPPUNIT_ASSERT_EQUAL(value_false, trueValue(negLit(0)));
-		CPPUNIT_ASSERT_EQUAL(value_false, falseValue(posLit(0)));
-		CPPUNIT_ASSERT_EQUAL(value_true, falseValue(negLit(0)));
+		CPPUNIT_ASSERT_EQUAL(value_true, trueValue(lit_true()));
+		CPPUNIT_ASSERT_EQUAL(value_false, trueValue(lit_false()));
+		CPPUNIT_ASSERT_EQUAL(value_false, falseValue(lit_true()));
+		CPPUNIT_ASSERT_EQUAL(value_true, falseValue(lit_false()));
 	}
 
 	void testLess() {
@@ -148,10 +144,7 @@ public:
 		CPPUNIT_ASSERT_EQUAL(true, negOne < two);
 		CPPUNIT_ASSERT_EQUAL(false, two < negOne);
 	}
-	void testAntecedentAssumptions() {
-		Antecedent::checkPlatformAssumptions();
-	}
-	
+
 	void testAntecedentNullPointer() {
 		Antecedent a;
 		Antecedent b( (Constraint*) 0 );
@@ -165,11 +158,11 @@ public:
 			void reason(Solver&, Literal, LitVec&) {}
 			Constraint* cloneAttach(Solver&) { return 0; }
 		};
-		
+
 		Constraint* c = new Con;
 		Antecedent a(c);
 		CPPUNIT_ASSERT_EQUAL(false, a.isNull());
-		CPPUNIT_ASSERT_EQUAL(Antecedent::generic_constraint, a.type());
+		CPPUNIT_ASSERT_EQUAL(Antecedent::Generic, a.type());
 		CPPUNIT_ASSERT_EQUAL(c, a.constraint());
 		c->destroy();
 	}
@@ -197,11 +190,11 @@ private:
 		Antecedent ap(p);
 		Antecedent aNotp(~p);
 		CPPUNIT_ASSERT_EQUAL(false, ap.isNull());
-		CPPUNIT_ASSERT_EQUAL(Antecedent::binary_constraint, ap.type());
+		CPPUNIT_ASSERT_EQUAL(Antecedent::Binary, ap.type());
 		CPPUNIT_ASSERT(p == ap.firstLiteral());
 
 		CPPUNIT_ASSERT_EQUAL(false, aNotp.isNull());
-		CPPUNIT_ASSERT_EQUAL(Antecedent::binary_constraint, aNotp.type());
+		CPPUNIT_ASSERT_EQUAL(Antecedent::Binary, aNotp.type());
 		CPPUNIT_ASSERT(~p == aNotp.firstLiteral());
 		return true;
 	}
@@ -212,14 +205,14 @@ private:
 		Antecedent ann(~p, ~q);
 
 		CPPUNIT_ASSERT_EQUAL(false, app.isNull());
-		CPPUNIT_ASSERT_EQUAL(Antecedent::ternary_constraint, app.type());
+		CPPUNIT_ASSERT_EQUAL(Antecedent::Ternary, app.type());
 		CPPUNIT_ASSERT_EQUAL(false, apn.isNull());
-		CPPUNIT_ASSERT_EQUAL(Antecedent::ternary_constraint, apn.type());
+		CPPUNIT_ASSERT_EQUAL(Antecedent::Ternary, apn.type());
 		CPPUNIT_ASSERT_EQUAL(false, anp.isNull());
-		CPPUNIT_ASSERT_EQUAL(Antecedent::ternary_constraint, anp.type());
+		CPPUNIT_ASSERT_EQUAL(Antecedent::Ternary, anp.type());
 		CPPUNIT_ASSERT_EQUAL(false, ann.isNull());
-		CPPUNIT_ASSERT_EQUAL(Antecedent::ternary_constraint, ann.type());
-		
+		CPPUNIT_ASSERT_EQUAL(Antecedent::Ternary, ann.type());
+
 		CPPUNIT_ASSERT(p == app.firstLiteral() && q == app.secondLiteral());
 		CPPUNIT_ASSERT(p == apn.firstLiteral() && ~q == apn.secondLiteral());
 		CPPUNIT_ASSERT(~p == anp.firstLiteral() && q == anp.secondLiteral());
@@ -229,4 +222,4 @@ private:
 };
 
 CPPUNIT_TEST_SUITE_REGISTRATION(LiteralTest);
-} } 
+} }
