@@ -1,18 +1,18 @@
-// 
+//
 // Copyright (c) 2009, Benjamin Kaufmann
-// 
-// This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
-// 
+//
+// This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/
+//
 // Clasp is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 2 of the License, or
 // (at your option) any later version.
-// 
+//
 // Clasp is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU General Public License
 // along with Clasp; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
@@ -41,18 +41,19 @@ class SharedClauseTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testSimplifyShared);
 
 	CPPUNIT_TEST(testCloneShared);
-	CPPUNIT_TEST_SUITE_END(); 
+	CPPUNIT_TEST_SUITE_END();
 public:
+	typedef ConstraintInfo ClauseInfo;
 	SharedClauseTest() {
-		a1 = posLit(ctx.addVar(Var_t::atom_var));
-		a2 = posLit(ctx.addVar(Var_t::atom_var));
-		a3 = posLit(ctx.addVar(Var_t::atom_var));
-		b1 = posLit(ctx.addVar(Var_t::body_var));
-		b2 = posLit(ctx.addVar(Var_t::body_var));
-		b3 = posLit(ctx.addVar(Var_t::body_var));
+		a1 = posLit(ctx.addVar(Var_t::Atom));
+		a2 = posLit(ctx.addVar(Var_t::Atom));
+		a3 = posLit(ctx.addVar(Var_t::Atom));
+		b1 = posLit(ctx.addVar(Var_t::Body));
+		b2 = posLit(ctx.addVar(Var_t::Body));
+		b3 = posLit(ctx.addVar(Var_t::Body));
 
 		for (int i = 6; i < 15; ++i) {
-			ctx.addVar(Var_t::atom_var);
+			ctx.addVar(Var_t::Atom);
 		}
 		ctx.startAddConstraints(10);
 	}
@@ -72,9 +73,9 @@ public:
 	void testPropRandomClauses() {
 		for (int i = 0; i != 100; ++i) {
 			SharedContext cc;
-			for (int j = 0; j < 12; ++j) { cc.addVar(Var_t::atom_var); }
+			for (int j = 0; j < 12; ++j) { cc.addVar(Var_t::Atom); }
 			cc.startAddConstraints(1);
-			
+
 			makeRandomClause( (rand() % 10) + 2 );
 			ClauseHead* c = createShared(cc, clLits, ClauseInfo());
 			cc.add(c);
@@ -103,22 +104,19 @@ public:
 	void testReasonBumpsActivityIfLearnt() {
 		makeLits(4, 0);
 		ctx.endInit();
-		ClauseInfo e(Constraint_t::learnt_conflict);
+		ClauseInfo e(Constraint_t::Conflict);
 		ClauseHead* c = createShared(ctx, clLits, e);
 		Solver& solver= *ctx.master();
 		solver.addLearnt(c, (uint32)clLits.size());
-		
+
 		solver.assume(~clLits[0]);
 		solver.propagate();
 		solver.assume(~clLits[1]);
 		solver.propagate();
-		solver.assume(~clLits[2]);
-		solver.propagate();
-		
-		CPPUNIT_ASSERT_EQUAL(true, solver.isTrue( clLits[3] ) );
 		uint32 a = c->activity().activity();
-		LitVec r;
-		solver.reason(clLits[3], r);
+		solver.assume(~clLits[2]);
+		solver.force(~clLits[3], Antecedent(0));
+		CPPUNIT_ASSERT_EQUAL(false, solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(a+1, c->activity().activity());
 	}
 
@@ -126,7 +124,7 @@ public:
 		makeLits(3, 2);
 		ClauseHead* c1 = createShared(ctx, clLits, ClauseInfo());
 		ctx.add(c1);
-		
+
 		ctx.addUnary(~clLits[4]);
 		ctx.addUnary(clLits[3]);
 		ctx.master()->propagate();
@@ -138,7 +136,7 @@ public:
 		makeLits(3, 3);
 		ClauseHead* c = createShared(ctx, clLits, ClauseInfo());
 		ctx.add(c);
-		
+
 		ctx.addUnary(~clLits[2]);
 		ctx.addUnary(~clLits[3]);
 		ctx.master()->propagate();
@@ -150,11 +148,11 @@ public:
 
 	void testSimplifyShared() {
 		makeLits(3, 3);
-		SharedLiterals* sLits   = SharedLiterals::newShareable(clLits, Constraint_t::learnt_conflict);
-		CPPUNIT_ASSERT(sLits->unique() && sLits->type() == Constraint_t::learnt_conflict && sLits->size() == 6);
+		SharedLiterals* sLits   = SharedLiterals::newShareable(clLits, Constraint_t::Conflict);
+		CPPUNIT_ASSERT(sLits->unique() && sLits->type() == Constraint_t::Conflict && sLits->size() == 6);
 		SharedLiterals* other   = sLits->share();
 		CPPUNIT_ASSERT(!sLits->unique());
-		
+
 		ctx.addUnary(~clLits[2]);
 		ctx.addUnary(~clLits[3]);
 		ctx.master()->propagate();
@@ -168,7 +166,7 @@ public:
 	void testCloneShared() {
 		makeLits(3, 2);
 		ClauseHead* c = createShared(ctx, clLits, ClauseInfo());
-		Solver& solver2 = ctx.addSolver();
+		Solver& solver2 = ctx.pushSolver();
 		ctx.endInit(true);
 		ClauseHead* clone = (ClauseHead*)c->cloneAttach(solver2);
 		LitVec lits;
@@ -185,7 +183,7 @@ public:
 		CPPUNIT_ASSERT(solver2.reason(clLits.back()) == clone);
 
 		clone->destroy(&solver2, true);
-		
+
 	}
 private:
 	SharedContext ctx;
@@ -196,7 +194,7 @@ private:
 		SharedLiterals* shared_lits = SharedLiterals::newShareable(lits, e.type());
 		return SharedLitsClause::newClause(*ctx.master(), shared_lits, e, &lits[0], false);
 	}
-	
+
 	void simplePropTest(ClauseHead* c) {
 		Solver& solver = *ctx.master();
 		solver.add(c);
@@ -275,8 +273,8 @@ private:
 	}
 	void makeRandomClause(int size) {
 		int pos = rand() % size + 1;
-		makeLits( pos, size - pos ); 
+		makeLits( pos, size - pos );
 	}
 };
 CPPUNIT_TEST_SUITE_REGISTRATION(SharedClauseTest);
-} } 
+} }
