@@ -12,8 +12,7 @@
 //  GNU General Public License for more details.
 //
 //  You should have received a copy of the GNU General Public License
-//  along with this file; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+//  along with this program. If not, see <http://www.gnu.org/licenses/>.
 //
 //
 // NOTE: ProgramOptions is inspired by Boost.Program_options
@@ -27,14 +26,48 @@
 #pragma warning (disable : 4200)
 #endif
 #include "value.h"
-#include "value_parser.h"
+#include "string_convert.h"
 #include "detail/notifier.h"
 #include "errors.h"
-#include <memory>
 namespace ProgramOptions { namespace detail {
 template <class T>
 struct Parser { typedef bool (*type)(const std::string&, T&); };
 } // end namespace detail
+using bk_lib::string_cast;
+///////////////////////////////////////////////////////////////////////////////
+// Enumeration Parser: string->int mapping
+///////////////////////////////////////////////////////////////////////////////
+struct ValueMappingBase : private std::vector<std::pair<const char*, int> > {
+	typedef std::vector<std::pair<const char*, int> > base_type;
+	using base_type::size;
+	using base_type::empty;
+	void       add(const char* strVal, int eVal){ push_back(value_type(strVal, eVal)); }
+	const int* get(const char* strVal) const    {
+		for (const_iterator it = begin(), end = this->end(); it != end; ++it) {
+			if (strcasecmp(strVal, it->first) == 0) { return &it->second; }
+		}
+		return 0;
+	}
+};
+template <class T>
+struct ValueMapping : ValueMappingBase {
+	typedef ValueMapping     this_type;
+	typedef ValueMappingBase base_type;
+	using base_type::get;
+	this_type& operator()(const char* strVal, T eVal){ base_type::add(strVal, static_cast<int>(eVal)); return *this; }
+	static ValueMapping& instance()                  { static ValueMapping m; return m; }
+	operator typename detail::Parser<T>::type()      { return &parse<T>; }
+	operator typename detail::Parser<int>::type()    { return &parse<int>; }
+	template <class U>
+	static bool parse(const std::string& value, U& out){ 
+		const ValueMappingBase& m = ValueMapping::instance();
+		if (const int* x = m.get(value.c_str())) { out = static_cast<U>(*x); return true; } 
+		return false; 
+	}
+private:
+	ValueMapping() {}
+};
+template <class T> ValueMapping<T>& values() { return ValueMapping<T>::instance(); }
 ///////////////////////////////////////////////////////////////////////////////
 // StoredValue - a typed value that writes to an existing variable
 ///////////////////////////////////////////////////////////////////////////////
@@ -59,6 +92,10 @@ protected:
 // NotifiedValue - a typed value that is created on demand and passed to a callback
 ////////////////////////////////////////////////////////////////////////////////////
 template <class T>
+struct DefaultCreator {
+	static T* create() { return new T(); }
+};
+template <class T>
 class NotifiedValue : public Value {
 public:
 	typedef typename detail::Parser<T>::type parser_type;
@@ -75,20 +112,18 @@ public:
 		return this;
 	}
 	bool doParse(const std::string& name, const std::string& value) {
-		bool ret;
-		T* pv = 0;
-		std::auto_ptr<T> holder;
+		T* pv;
+		detail::Owned<T> exit = { (pv = 0) };
 		if (this->hasProperty(Value::property_location)) {
-			pv  = value_.address;
+			pv = value_.address;
 		}
 		else {
-			holder.reset(value_.create());
-			pv = holder.get();
+			exit.obj = (pv = value_.create());
 		}
-		ret = this->parser_(value, *pv);
+		bool ret = this->parser_(value, *pv);
 		if (ret && notify_.notify(name, pv)) {
 			this->storeTo(*pv);
-			holder.release();
+			exit.obj = 0;
 		}
 		return ret;
 	}
@@ -121,10 +156,22 @@ protected:
 #define LIT_TO_STRING_X(lit) #lit
 //! stringifies a literal like 1 or 23.0
 #define LIT_TO_STRING(lit) LIT_TO_STRING_X(lit)
-enum FlagAction {
-	store_false= 0,
-	store_true = 1
+struct FlagAction {
+	typedef detail::Parser<bool>::type parser_t;
+	static inline bool store_true(const std::string& v, bool& b) { 
+		if (v.empty()) { return (b = true); }
+		return string_cast<bool>(v, b);
+	}
+	static inline bool store_false(const std::string& v, bool& b) { 
+		bool temp;
+		return store_true(v, temp) && ((b = !temp), true);
+	}
+	enum Action { act_store_true, act_store_false } act;
+	FlagAction(Action a) : act(a) {}
+	parser_t parser() const { return act == act_store_true ? store_true : store_false; }
 };
+static const FlagAction store_true(  (FlagAction::act_store_true) );
+static const FlagAction store_false( (FlagAction::act_store_false));
 
 /*!
  * Creates a value that is bound to an existing variable.
@@ -136,11 +183,11 @@ enum FlagAction {
  *           type T must provide an operator>>(std::istream&, T&).
  */
 template <class T>
-inline StoredValue<T>* storeTo(T& v, typename detail::Parser<T>::type p = &DefaultParser<T>::parse) {
+inline StoredValue<T>* storeTo(T& v, typename detail::Parser<T>::type p = &string_cast<T>) {
 	return new StoredValue<T>(v, p);
 }
 inline StoredValue<bool>* flag(bool& b, FlagAction x = store_true) { 
-	return static_cast<StoredValue<bool>*>(storeTo(b, FlagStr::parser(x))->flag());
+	return static_cast<StoredValue<bool>*>(storeTo(b, x.parser())->flag());
 }
 
 /*!
@@ -162,16 +209,16 @@ inline StoredValue<bool>* flag(bool& b, FlagAction x = store_true) {
  * \see OptionGroup::addOptions()
  */
 template <class T, class ParamT>
-inline NotifiedValue<T>* notify(ParamT* p0, typename detail::Notify<const T*, ParamT>::type nf, typename detail::Parser<T>::type parser = &DefaultParser<T>::parse) {
+inline NotifiedValue<T>* notify(ParamT* p0, typename detail::Notify<const T*, ParamT>::type nf, typename detail::Parser<T>::type parser = &string_cast<T>) {
 	return new NotifiedValue<T>(&DefaultCreator<T>::create, detail::Notifier<const T*>(p0, nf), parser);
 }
 template <class T, class ParamT>
-inline NotifiedValue<T>* storeNotify(T& obj, ParamT* p0, typename detail::Notify<const T*, ParamT>::type nf, typename detail::Parser<T>::type parser = &DefaultParser<T>::parse) {
+inline NotifiedValue<T>* storeNotify(T& obj, ParamT* p0, typename detail::Notify<const T*, ParamT>::type nf, typename detail::Parser<T>::type parser = &string_cast<T>) {
 	return notify<T>(p0, nf, parser)->storeTo(obj);
 }
 template <class ParamT>
 inline NotifiedValue<bool>* flag(ParamT* p0, typename detail::Notify<const bool*, ParamT>::type nf, FlagAction a = store_true) {
-	return static_cast<NotifiedValue<bool>*>(notify<bool>(p0, nf, FlagStr::parser(a))->flag());
+	return static_cast<NotifiedValue<bool>*>(notify<bool>(p0, nf, a.parser())->flag());
 }
 
 /*!
